/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include "exynth.h"
#include "timer.h"
#include "font.h"
#include "server/fb.h"
#include "input/input.h"

//#define DEBUG
#include "debug.h"

static s_exynth_t *g_exynth = NULL;

/* Initialize exynth library. Start input thread
 *
 * This must be called first before using any exynth functions! */
int s_exynth_init()
{
	if (g_exynth != NULL) return -1;
	g_exynth = calloc(1,sizeof(s_exynth_t));
	if (g_exynth == NULL) return -2;

	if (s_font_cache_init()) goto out1;
	if (s_timers_init() != 0) goto out1;
	if (s_server_surface_init() != 0) goto out1;
	if (fb_init() != 0) goto out2;
	if (s_server_surface_create() != 0) goto out3;
	s_list_sinit(&(g_exynth->exitlist));

	pthread_mutexattr_t attr;
	pthread_mutexattr_init(&attr);
	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
	if (pthread_mutex_init(&g_exynth->mutex, &attr) != 0) goto out4;
	pthread_mutexattr_destroy(&attr);

	if (s_input_init() != 0) goto out5;

	return 0;
out5:
	pthread_mutex_destroy(&g_exynth->mutex);
out4:
out3:
	fb_uninit();
out2:
	s_server_surface_uninit();
out1:
	free(g_exynth);
	return -3;
}

// De-initialize exynth library
int s_exynth_uninit()
{
	s_input_uninit();
	fb_uninit();
	s_server_surface_uninit();
	pthread_mutex_destroy(&g_exynth->mutex);
	free(g_exynth);
	return 0;
}

// -------------------------------------------------------------------
// Exynth mutex. The only mutex, must be taken when any calls to
// GUI functions are made from other threads. Is taken by exynth
// when main loop is non-idle.

int s_exynth_timedlock(const struct timespec *abs_timeout)
{
	int ret;
	ret = pthread_mutex_timedlock(&(g_exynth->mutex), abs_timeout);
	debug("EXYNTH MUTEX LOCK\n");
	return ret;
}

int s_exynth_lock()
{
	int ret;
	ret = pthread_mutex_lock(&(g_exynth->mutex));
	debug("EXYNTH MUTEX LOCK\n");
	return ret;
}

int s_exynth_unlock()
{
	debug("EXYNTH MUTEX UNLOCK\n");
	return pthread_mutex_unlock(&(g_exynth->mutex));
}

s_exynth_t* get_exynth(void)
{
	return g_exynth;
}

int s_exynth_set_idle_func(int(*f)(void))
{
	if (g_exynth->idle_func != NULL && f != NULL) return -1;
	g_exynth->idle_func = f;
	return 0;
}
