/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef IMAGE_H_
#define IMAGE_H_

#if defined(__cplusplus)
	extern "C" {
#endif

typedef struct s_image_s s_image_t;

struct s_image_s {
	/** layer w */
	int w;
	/** layer h */
	int h;
	/** rgba data of image RED | GREEN | BLUE | ALPHA*/
	unsigned int *rgba;
};


int s_image_init (s_image_t **img);

/** @brief frees memory used for buf, and sets it to NULL.
  *
  * @param *img - the image
  * @returns 0 on success, 1 on error.
  */
void s_image_free_rgba (s_image_t *img);

/** @brief frees memory used for handler, and sets it to NULL.
  *
  * @param *img - the image
  * @returns 0 on success, 1 on error.
  */
int s_image_uninit (s_image_t *img);

/** @brief initializes image struct
  *
  * @param **lyr - address of the pointer to image struct
  * @returns 0 on success, 1 on error.
  */
int s_image_img (const char *file, s_image_t *img);

/* image_png.c */

/** @brief checks if the file is a png
  *
  * @param *file - exact path to the file
  * @returns 0 on success, 1 on error.
  */
int s_image_png_is (const char *file);

/** @brief fills the image struct
  *
  * @param *file - exact path to the file
  * @param *img  - the image
  * @returns 0 on success, 1 on error.
  */
int s_image_png (const char *file, s_image_t *img);

#if defined(__cplusplus)
	}
#endif

#endif /* IMAGE_H_ */
