/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <linux/input.h>
#include <sys/ioctl.h>

#include "../exynth.h"
#include "input.h"

//#define DEBUG
#include "../debug.h"

static int fdKeypad = -1;
static int suppress = 0;

/* Initialize keypad driver, return fd */
int input_keypad_init (void)
{
	char devFile[32];
	unsigned char bitMask[EV_MAX/8 + 1];
	int i;

	debug("input_keypad_init()\n");

	// cycle through event interfaces to discover a keypad
	for(i = 0; i < 32; ++i)
	{
		sprintf(devFile, "/dev/input/event%d", i);
		fdKeypad = open(devFile, O_NONBLOCK | O_RDWR);

		if(fdKeypad >= 0)
		{
			if(ioctl(fdKeypad, EVIOCGBIT(0, sizeof(bitMask)), bitMask) < 0)
			{
				close(fdKeypad);
				fdKeypad = -1;
			}

			// detect a keypad driver by presense of the KEY bit
			if(bitMask[EV_KEY/8] & (1<<(EV_KEY % 8)))
			{
				debug("input_keypad_init: using %s\n", devFile);
				return fdKeypad;
			}
			else
			{
				close(fdKeypad);
				fdKeypad = -1;
			}
		}
	}

	debug("input_keypad_init: no keypads found\n");
	return -1;
}

/* Un-initialize keypad driver */
void input_keypad_uninit (void)
{
	debug("input_keypad_uninit()\n");

	if(fdKeypad >= 0 )
	{
		close(fdKeypad);
		fdKeypad = -1;
	}
}

/* Handle keypad input */
void input_keypad_update (void)
{
	struct input_event eventData;
	int readBytes;
	debug("input_keypad_update()\n");

	readBytes = read(fdKeypad, &eventData, sizeof(eventData));
	if(readBytes != sizeof(eventData)) return;

	if(eventData.type == EV_KEY)
		input_keyboard_parse_submit(eventData.code, eventData.value);
}

/* Submit keypress */
void input_keyboard_parse_submit(int key, int value)
{
	s_exynth_t *e = get_exynth();
	int top, i;
	debug("input_keyboard_parse_submit: key %d, value %d\n", key, value);

	top = e->pri_top;
	s_event_t ev;
	ev.type = KEYBD_EVENT;
	ev.data.keyboard.keycode = key;
	ev.data.keyboard.handled = 0;
	if (value) ev.type |= KEYBD_PRESSED;
	else ev.type |= KEYBD_RELEASED;

	s_exynth_t *ex = get_exynth();
	if (ex->idle_func) {
		if (ex->idle_func()) {
			suppress = 1;
			return;
		}
	}
	if (suppress != 0 && !value) return;
	suppress = 0;

	for (i=1; i<S_CLIENTS_MAX && !ev.data.keyboard.handled; i++) {
		if (e->clients[i] == NULL) continue;
		if (e->clients[i]->type & WINDOW_INPUT) {
			s_window_event(e->clients[i], &ev);
		}
	}
	for (i=1; i<S_CLIENTS_MAX && !ev.data.keyboard.handled; i++) {
		if (e->clients[i] == NULL) continue;
		if (i == top && !(e->clients[i]->type & WINDOW_INPUT)) {
			s_window_event(e->clients[i], &ev);
		}
	}
}
