/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <linux/input.h>
#include <sys/ioctl.h>

#include "../exynth.h"
#include "../event.h"
#include "input.h"
#include "../window.h"

static int fdTouch = -1;
static int minX=0, maxX=0, minY=0, maxY=0;
static int touchX=0, touchY=0, touchPressure=0, touchButton=0;
static int pButtons=0;
static int suppress = 0;

/* Initialize touchscreen */
int input_touch_init ()
{
	char devFile[32];
	unsigned char bitMask[EV_MAX/8 + 1];
	struct input_absinfo absInfo;
	int i;

	// cycle through event interfaces to discover touch screen
	for(i = 0; i < 32; ++i)
	{
		sprintf(devFile, "/dev/input/event%d", i);
		fdTouch = open(devFile, O_NONBLOCK | O_RDWR);

		if(fdTouch >= 0)
		{
			if(ioctl(fdTouch, EVIOCGBIT(0, sizeof(bitMask)), bitMask) < 0)
			{
				close(fdTouch);
				fdTouch = -1;
			}
			
			// detect a touchscreen driver by presense of the KEY bit
			if(bitMask[EV_ABS/8] & (1<<(EV_ABS % 8))) 
			{
				ioctl(fdTouch, EVIOCGABS(ABS_X), &absInfo);
				minX = absInfo.minimum;
				maxX = absInfo.maximum;

				ioctl(fdTouch, EVIOCGABS(ABS_Y), &absInfo);
				minY = absInfo.minimum;
				maxY = absInfo.maximum;
				
				return fdTouch;
			}
			else
			{
				close(fdTouch);
				fdTouch = -1;
			}
		}
	}

	return -1;
}

/* Un-initialize touchscreen */
void input_touch_uninit (void)
{
	if(fdTouch >= 0 )
	{
		close(fdTouch);
		fdTouch = -1;
	}
}

/* Read touchscreen input */
void input_touch_update ()
{
	s_exynth_t *e = get_exynth();
	struct input_event eventData;
	int readBytes;
	int x,y,b;
	
	while(1)
	{
		readBytes = read(fdTouch, &eventData, sizeof(eventData));
		if(readBytes != sizeof(eventData)) return;

		if(eventData.type == EV_ABS) switch(eventData.code)
		{
		case ABS_X:
			
			touchX = eventData.value;
			break;
			
		case ABS_Y:
			
			touchY = eventData.value;
			break;
			
		case ABS_PRESSURE:
			
			touchPressure = eventData.value;
			break;
		}
		else if(eventData.type == EV_KEY) switch(eventData.code)
		{
		case BTN_TOUCH:
			
			touchButton = eventData.value;
			break;
		}
		else if(eventData.type == EV_SYN) break;
	}

	x = ((touchX - minX) * e->root_surface->width) / (maxX - minX);
	y = e->root_surface->height - ((touchY - minY) * e->root_surface->height) / (maxY - minY);
	b = touchButton ? MOUSE_LEFTBUTTON : 0;
	
	input_touch_parse(x, y, b);

	return;
}

/* Parse touchscreen event */
void input_touch_parse (int x, int y, int b)
{
	long long time;
	s_event_t event;
	memset(&event,0,sizeof(s_event_t));

	s_exynth_t *e = get_exynth();
	if (e->idle_func) {
		if (e->idle_func()) {
			suppress = 1;
			return;
		}
	}
	if (suppress != 0) {
		if (b == 0) suppress = 0;
		return;
	}

	time = s_gettimeofday();
	event.type = MOUSE_EVENT;

	event.data.mouse.x = x;
	event.data.mouse.y = y;
	event.data.mouse.buttons = b;

	if (event.data.mouse.buttons > pButtons) {
		event.type |= MOUSE_PRESSED;
		event.data.mouse.press = (event.data.mouse.buttons & ~(pButtons));
	} else if (event.data.mouse.buttons < pButtons) {
		event.type |= MOUSE_RELEASED;
	} else {
		if (event.data.mouse.buttons == 0) {
			event.type |= MOUSE_OVER;
		} else {
			event.type |= (MOUSE_OVER | MOUSE_HINT);
		}
	}

	event.data.mouse.time = time;

	pButtons = b;

	input_touch_submit(&event);
}

/* Handle touchscreen event */
void input_touch_submit (s_event_t *event)
{
	int id;
	int oid;
	s_exynth_t *e = get_exynth();

	if (event->data.mouse.x < 0 || event->data.mouse.x >= e->root_surface->width) return;
	if (event->data.mouse.y < 0 || event->data.mouse.y >= e->root_surface->height) return;

	id = *(e->root_surface->matrix + (event->data.mouse.y * e->root_surface->width) + event->data.mouse.x);
	oid = e->mouse_xyid;
	e->mouse_xyid = id;

	// send the event to the topmost window at this position
	// if the window has changed, send MOUSE_ENTERED event, too
	if (id != oid) {
		event->type |= MOUSE_ENTERED;
		event->type &= ~MOUSE_EXITED;
	}
	if (e->clients[id]) {
		s_window_event(e->clients[id], event);
	}

	// send to the window where the mouse was clicked
	// if we got a release elsewhere
	if (event->type & MOUSE_RELEASED && e->mouse_xyid_oldclick != -1) {
		if (id != e->mouse_xyid_oldclick) {
			if (e->clients[e->mouse_xyid_oldclick]) {
				s_window_event(e->clients[e->mouse_xyid_oldclick], event);
			}
		}
		e->mouse_xyid_oldclick = -1;
	}
	if (event->type & MOUSE_PRESSED) {
		e->mouse_xyid_oldclick = id;
	}

	// if the mouse has moved to another window, send MOUSE_EXITED event to previous
	if (id != oid) {
		event->type &= ~MOUSE_ENTERED;
		event->type |= MOUSE_EXITED;
		if (e->clients[oid]) {
			s_window_event(e->clients[oid], event);
		}
	}
}
