/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include "exynth.h"
#include "list.h"

int s_list_init (s_list_t **li)
{
	(*li) = (s_list_t *) malloc(sizeof(s_list_t));
	if ((*li) == NULL) {
		return -1;
	}
	(*li)->nb_elt = 0;
	(*li)->node = NULL;
	return 0;
}

int s_list_uninit (s_list_t *li)
{
	free(li);
	return 0;
}

int s_list_sinit (s_list_t *li)
{
	li->nb_elt = 0;
	li->node = NULL;
	return 0;
}

int s_list_suninit (s_list_t *li)
{
	return 0;
}


int s_list_eol (s_list_t *li, int i)
{
	if (li == NULL) {
		return 1;
	}

	if ((i >= 0) && (i < li->nb_elt)) {
		return 0;
	}

	/* end of list */
	return 1;
}

void * s_list_get (s_list_t *li, int pos)
{
	int i = 0;
	s_list_node_t *ntmp;

	if ((li == NULL) || (pos < 0) || (pos >= li->nb_elt)) {
		/* element does not exist */
		return NULL;
	}

	/* exist because nb_elt > 0 */
	ntmp = li->node;

	while (pos > i) {
		i++;
		ntmp = ntmp->next;
	}

	return ntmp->element;
}

int s_list_remove (s_list_t *li, int pos)
{
	int i = 0;
	s_list_node_t *ntmp;

	if ((li == NULL) || (pos < 0) || (pos >= li->nb_elt)) {
		/* element does not exist */
		return -1;
	}

	ntmp = li->node;

	if ((pos == 0)) {
		/* special case  */
		li->node = ntmp->next;
		li->nb_elt--;
		free(ntmp);
		return li->nb_elt;
	}

	pos--;
	while (pos > i) {
		i++;
		ntmp = ntmp->next;
	}

	{
		s_list_node_t *remnode;

		remnode = ntmp->next;
		ntmp->next = ntmp->next->next;
		free(remnode);
		li->nb_elt--;
	}

	return li->nb_elt;
}

int s_list_add (s_list_t *li, void *el, int pos)
{
	int i = 0;
	s_list_node_t *ntmp;

	if (li == NULL) {
		return -1;
	}

	if ((pos == -1) || (pos >= li->nb_elt)) {
		/* insert at the end  */
		pos = li->nb_elt;
	}

	if (pos == 0) {
		s_list_node_t *new = (s_list_node_t *) malloc(sizeof(s_list_node_t));
		if (!new) return -2;
		new->element = el;
		new->next = li->node;
		li->node = new;
		li->nb_elt++;
		return li->nb_elt;
	}

	ntmp = li->node;
	pos -= 1;
	while (pos > i) {
		i++;
		/* when pos > i next node exist  */
		ntmp = (s_list_node_t *) ntmp->next;
	}

	/* here pos == i so next node is where we want to insert new node */
	{
		s_list_node_t *new = (s_list_node_t *) malloc(sizeof(s_list_node_t));
		if (!new) return -2;
		new->element = el;
		new->next = ntmp->next;
		ntmp->next = new;
		li->nb_elt++;
	}

	return li->nb_elt;
}

void * s_list_find (s_list_t *li, void *node, int (*cmp_func) (void *, void *))
{
	s_list_node_t *ntmp;

	if (li == NULL) return NULL;

	ntmp = li->node;
	while (ntmp) {
		if (cmp_func(node, ntmp->element) == 0)
			return ntmp->element;
		ntmp = ntmp->next;
	}

	return NULL;
}

int s_list_find_pos (s_list_t *li, void *node, int (*cmp_func) (void *, void *))
{
	int i = 0;
	s_list_node_t *ntmp;

	if (li == NULL) return -1;

	ntmp = li->node;
	while (ntmp) {
		if (cmp_func(node, ntmp->element) == 0)
			return i;
		ntmp = ntmp->next;
		i++;
	}

	return -1;
}

int s_list_get_pos (s_list_t *li, void *node)
{
	int i = 0;
	s_list_node_t *ntmp;

	if (li == NULL) return -1;

	ntmp = li->node;
	while (ntmp) {
		if (node == ntmp->element)
			return i;
		ntmp = ntmp->next;
		i++;
	}

	return -1;
}

/* List iteration */
s_list_node_t *s_list_first(s_list_t *list)
{
	if (!list) return NULL;
	return list->node;
}

s_list_node_t *s_list_next(s_list_node_t *node)
{
	if (!node) return NULL;
	return node->next;
}

void *s_list_element(s_list_node_t *node)
{
	if (!node) return NULL;
	return node->element;
}

int s_list_del_next(s_list_t *li, s_list_node_t *node)
{
	s_list_node_t *ntmp;
	if (!li) return -1;

	// del first?
	if (node == NULL) {
		ntmp = li->node;
		li->node = ntmp->next;
		li->nb_elt--;
		free(ntmp);
		return li->nb_elt;
	}

	if (node->next == NULL) return -2;

	ntmp = node->next;
	node->next = ntmp->next;
	li->nb_elt--;
	free(ntmp);
	return li->nb_elt;
}
