/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "exynth.h"
#include "rect.h"

#define MIN(a, b)	((a) < (b) ? (a) : (b))
#define MAX(a, b)	((a) > (b) ? (a) : (b))

int s_rect_merge (s_rect_t *r1, s_rect_t *r2, s_rect_t *r)
{
	int x31;
	int x32;
	int y31;
	int y32;

	x31 = MIN(r1->x, r2->x);
	x32 = MAX(r1->x + r1->w - 1, r2->x + r2->w - 1);

	y31 = MIN(r1->y, r2->y);
	y32 = MAX(r1->y + r1->h - 1, r2->y + r2->h - 1);
	
	r->x = x31;
	r->y = y31;
	r->w = x32 - x31 + 1;
	r->h = y32 - y31 + 1;
	
	return 0;
}

int s_rect_intersect (s_rect_t *r1, s_rect_t *r2, s_rect_t *r)
{
	int x31;
	int x32;
	int y31;
	int y32;

	x31 = MAX(r1->x, r2->x);
	x32 = MIN(r1->x + r1->w - 1, r2->x + r2->w - 1);

	y31 = MAX(r1->y, r2->y);
	y32 = MIN(r1->y + r1->h - 1, r2->y + r2->h - 1);

	if ((x31 > x32) ||
	    (y31 > y32)) {
		return -1;
	}

	r->x = x31;
	r->y = y31;
	r->w = x32 - x31 + 1;
	r->h = y32 - y31 + 1;

	return 0;
}

int s_rect_clip_virtual (s_surface_t *surface, int x, int y, int w, int h, s_rect_t *coor)
{
	s_rect_t thip;
	s_rect_t clip;

	thip.x = x;
	thip.y = y;
	thip.w = w;
	thip.h = h;

	clip.x = 0;
	clip.y = 0;
	clip.w = surface->width;
	clip.h = surface->height;

	if (s_rect_intersect(&thip, &clip, coor)) {
		return -1;
	}
	return 0;
}

int s_rect_clip_real (s_surface_t *surface, int x, int y, int w, int h, s_rect_t *coor)
{
        s_rect_t that;
	s_rect_t thip;
	s_rect_t cliv;
	s_rect_t clir;
	
        thip.x = x;
	thip.y = y;
	thip.w = w;
	thip.h = h;

	cliv.x = 0;
	cliv.y = 0;
	cliv.w = MIN(surface->buf.w, surface->width);
	cliv.h = MIN(surface->buf.h, surface->height);

	clir.x = 0;
	clir.y = 0;
	clir.w = surface->linear_buf_width;
	clir.h = surface->linear_buf_height;

	if (s_rect_intersect(&thip, &cliv, &that)) {
		return -1;
	}
	that.x += surface->buf.x;
	that.y += surface->buf.y;
	if (s_rect_intersect(&clir, &that, coor)) {
		return -1;
	}
	coor->x -= surface->buf.x;
	coor->y -= surface->buf.y;
	return 0;
}

