/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <sys/ioctl.h>
#include <sys/mman.h>
#include <linux/fb.h>
#include <fcntl.h>
#include <unistd.h>

#include "../exynth.h"

//#define DEBUG
#include "../debug.h"

int fb_fd = -1;

/* Un-initialize framebuffer */
void fb_uninit (void)
{
	s_exynth_t *e = get_exynth();
	debug("fb_uninit()\n");
	munmap(e->root_surface->linear_buf, e->root_surface->linear_mem_size);
	e->root_surface->linear_buf = NULL;
	close(fb_fd);
	fb_fd = -1;
}

/* Initialize framebuffer, read root surface settings */
int fb_init (void)
{
	s_exynth_t *e = get_exynth();
	struct fb_var_screeninfo v_scr;
	struct fb_fix_screeninfo f_scr;

	debug("fb_init()\n");
	if ((fb_fd = open(FB_DEVICE_NAME, O_RDWR)) == -1) {
		debug("fb_init: Unable to open device (%s)\n", FB_DEVICE_NAME);
		return -1;
	}
	if (ioctl(fb_fd, FBIOGET_VSCREENINFO, &v_scr) == -1) {
		debug("fb_init: Unable to get variable screen info\n");
		close(fb_fd);
		fb_fd = -1;
		return -2;
	}
	if (ioctl(fb_fd, FBIOGET_FSCREENINFO, &f_scr) == -1) {
		debug("fb_init: Unable to get fixed screen info\n");
		close(fb_fd);
		fb_fd = -1;
		return -3;
	}

	e->root_surface->width = v_scr.xres;
	e->root_surface->height = v_scr.yres;
	e->root_surface->bitsperpixel = v_scr.bits_per_pixel;
	e->root_surface->bytesperpixel = ((v_scr.bits_per_pixel -1) / 8) + 1;
	e->root_surface->blueoffset = v_scr.blue.offset;
	e->root_surface->greenoffset = v_scr.green.offset;
	e->root_surface->redoffset = v_scr.red.offset;
	e->root_surface->bluelength = v_scr.blue.length;
	e->root_surface->greenlength = v_scr.green.length;
	e->root_surface->redlength = v_scr.red.length;
	e->root_surface->colors = 1<<(v_scr.bits_per_pixel);

	debug("fb_init: %d x %d RGB %d %d %d\n", e->root_surface->width,
		e->root_surface->height, e->root_surface->redlength,
		e->root_surface->greenlength, e->root_surface->bluelength);

	e->root_surface->linear_mem_size = f_scr.smem_len;

	e->root_surface->linear_buf = mmap((caddr_t) 0, e->root_surface->linear_mem_size,
						                               PROT_WRITE | PROT_READ,
						                               MAP_SHARED,
						                               fb_fd,
						                               0);
	if (e->root_surface->linear_buf == MAP_FAILED) {
		debug("fb_init: mmap failed\n");
		close(fb_fd);
		fb_fd = -1;
		return -4;
	}

	debug("fb_init: linear buf mmap'ed at %p\n", e->root_surface->linear_buf);
	return 0;
}
