/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "../exynth.h"
#include "../rect.h"

#include "../driver.h"

/* Surface matrix - one byte per pixel matrix that describes which window
 * owns a pixel */

/* find which windows are present on this rect */
void s_server_surface_matrix_find (s_rect_t *coor, unsigned char *dm)
{
	int i;
	int h;
	int w;
	int w_;
	s_rect_t clip;
	s_rect_t inter;
	unsigned char *tmp;
	s_exynth_t *e = get_exynth();

	clip.x = 0;
	clip.y = 0;
	clip.w = e->root_surface->width;
	clip.h = e->root_surface->height;
	tmp = e->root_surface->matrix;

	if (s_rect_intersect(&clip, coor, &inter)) {
		return;
	}

	int top = e->pri_top;
	if (top > 0) {
		s_rect_t *tr = &e->clients[top]->surface->buf;
		if (tr->x <= inter.x && tr->y <= inter.y &&
				tr->x + tr->w >= inter.x + inter.w &&
				tr->y + tr->h >= inter.y + inter.h) {
			dm[top/8] |= (1<<(top%8));
			return;
		}
	}

	h = inter.h;
	w = inter.w;
	w_ = clip.w - inter.w;
	tmp += ((inter.y * clip.w) + inter.x);

	int oi = -1;
	while (h--) {
		w = inter.w;
		while (w--) {
			i = *tmp;
			tmp++;

			if (i == oi) continue;
			oi = i;

			if (i >= S_CLIENTS_MAX) continue;
			dm[i/8] |= (1<<(i%8));
		}
		tmp += w_;
	}
	return;
}

/* fill rect in matrix with id */
void s_server_surface_matrix_add_id (int id, s_rect_t *coor)
{
	int h;
	int w;
	int w_;
	s_rect_t clip;
	s_rect_t inter;
	unsigned char *tmp;
	s_exynth_t *e = get_exynth();

	clip.x = 0;
	clip.y = 0;
	clip.w = e->root_surface->width;
	clip.h = e->root_surface->height;
	tmp = e->root_surface->matrix;

	if (s_rect_intersect(coor, &clip, &inter)) {
		return;
	}

	h = inter.h;
	w = inter.w;
	w_ = clip.w - w;
	tmp += ((inter.y * clip.w) + inter.x);

	while (h--) {
		w = inter.w;
		while (w--) {
			*tmp++ = id;
		}
		tmp += w_;
	}
}

/* fill intersect(rect,window[id]) with id */
void s_server_surface_matrix_add (int id, s_rect_t *coor)
{
	s_rect_t inter;
	s_exynth_t *e = get_exynth();
	if (id < 0 || id > S_CLIENTS_MAX) return;
	if (e->clients[id] == NULL) return;
	if (s_rect_intersect(coor, &e->clients[id]->surface->buf, &inter)) {
		return;
	}
	s_server_surface_matrix_add_id(id, &inter);
}

/* fill rect with zeroes */
void s_server_surface_matrix_del_coor (s_rect_t *coor)
{
	s_server_surface_matrix_add_id(0, coor);
}
