/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "../exynth.h"
#include "server.h"

//#define DEBUG
#include "../debug.h"

// ------------------------------------------------------------------
// Priority array - window stack from bottom to top
// Actually, there are 3 arrays in one, separated by window
// alwaysontop parameter.

// move a window to the top of the priority array
int s_server_pri_arr_focus(int id)
{
	int i;
	s_exynth_t *e = get_exynth();
	// find first free slot or the same ID
	for (i=0; i<S_CLIENTS_MAX; i++) {
		if (e->pri[i] == id) break;
		if (e->pri[i] == 0) break;
	}
	// if the same ID, shift it off from list
	if (e->pri[i] == id) {
		for (; i<S_CLIENTS_MAX-1; i++) {
			e->pri[i] = e->pri[i+1];
			if (e->pri[i] == 0) break;
		}
	}
	// add the ID to the end of the list
	if (i < S_CLIENTS_MAX) {
		e->pri[i] = id;
		return i;
	}
	return -1;
}
// find topmost visible window
void s_server_update_top_pri()
{
	s_exynth_t *e = get_exynth();
	int i;
	int top[3] = {-1,-1,-1}; //alwaysonbottom, normal, alwaysontop
	e->pri_top = 0;
	for (i=0;i<S_CLIENTS_MAX; i++) {
		int ontop;
		int id = e->pri[i];
		if (id == 0) break;
		if (e->clients[id] == NULL) continue;
		if (e->clients[id]->visible == 0) continue;
		ontop = ((e->clients[id]->alwaysontop)%2)+1;
		top[ontop] = id;
	}
	if (top[2] != -1) e->pri_top = top[2];
	else if (top[1] != -1) e->pri_top = top[1];
	else if (top[0] != -1) e->pri_top = top[0];
	else e->pri_top = 0;
}

// remove window from priority array
void s_server_pri_arr_del(int id)
{
	int i;
	s_exynth_t *e = get_exynth();
	// find first free slot or the same ID
	for (i=0; i<S_CLIENTS_MAX; i++) {
		if (e->pri[i] == id) break;
		if (e->pri[i] == 0) return;
	}
	// shift ID off from list
	for (; i<S_CLIENTS_MAX-1; i++) {
		e->pri[i] = e->pri[i+1];
		if (e->pri[i] == 0) break;
	}
	// overwrite the last position that won't get shifted
	e->pri[S_CLIENTS_MAX-1] = 0;
}

// ------------------------------------------------------------------
// Matrix

// add/move a window to the top of the stack
void s_server_pri_focus_prepare(int id)
{
	s_exynth_t *e = get_exynth();
	s_rect_t *rect;
	debug("s_server_pri_focus_prepare(%d)\n", id);
	if (e->clients[id] == NULL) return;

	rect = &e->clients[id]->surface->buf;

	e->clients[id]->pri = s_server_pri_arr_focus(id);
	debug("s_server_pri_focus_prepare: pri %d\n", e->clients[id]->pri);
}

// add/move a window to the top of the stack, update matrix
void s_server_pri_focus(int id)
{
	s_exynth_t *e = get_exynth();
	s_rect_t *rect;
	debug("s_server_pri_focus(%d)\n", id);
	if (e->clients[id] == NULL) return;

	rect = &e->clients[id]->surface->buf;

	if (e->clients[id]->pri == -1)
		e->clients[id]->pri = s_server_pri_arr_focus(id);

	s_server_update_top_pri();
	s_server_pri_update(rect);
	s_server_pri_notify();
}

// move window on screen
void s_server_pri_move(int id, s_rect_t *old)
{
	s_exynth_t *e = get_exynth();
	s_rect_t tmp, diff;
	s_rect_t *new;
	if (e->clients[id] == NULL) return;
	if (e->clients[id]->visible == 0) return;

	new = &e->clients[id]->surface->buf;
	debug("s_server_pri_move(%d)\n", id);

	// diff = old - new
	if (s_rect_intersect(old, new, &diff)) {
		s_server_pri_update(old);
		s_server_pri_update(new);
		return;
	}

	/* aaccccbb
	   aa....bb
	   aa....bb
	   aaddddbb
	 */

	//A: {old->x, old->y, diff.x - old->x, old->h};
	tmp = *old;
	tmp.w = diff.x - old->x;
	s_server_pri_update(&tmp);

	//B: {diff.x + diff.w, old->y, (old->x + old->w) - (diff.x + diff.w), old->h};
	tmp.x = diff.x + diff.w;
	tmp.w = (old->x + old->w) - (diff.x + diff.w);
	s_server_pri_update(&tmp);

	//C: {diff.x, old->y, diff.w, diff.y - old->y};
	tmp = diff;
	tmp.y = old->y;
	tmp.h = diff.y - old->y;
	s_server_pri_update(&tmp);

	//D: {diff.x, diff.y + diff.h, diff.w, (old->y + old->h) - (diff.y + diff.h)};
	tmp.y = diff.y + diff.h;
	tmp.h = (old->y + old->h) - (diff.y + diff.h);
	s_server_pri_update(&tmp);

	// new rectangle
	s_server_pri_update(new);
}

// delete a window from screen
void s_server_pri_del(int id)
{
	s_rect_t *rect;
	s_exynth_t *e = get_exynth();
	debug("s_server_pri_del(%d)\n", id);
	if (e->clients[id] == NULL) return;

	rect = &e->clients[id]->surface->buf;

	s_server_pri_arr_del(id);
	e->clients[id]->pri = 0;

	s_server_update_top_pri();

	s_server_pri_update(rect);
	s_server_pri_notify();
}

// re-calculate a rect and send expose
void s_server_pri_update(s_rect_t *rect)
{
	debug("s_server_pri_update(x%d y%d w%d h%d)\n",
			rect->x, rect->y, rect->w, rect->h);
	if (rect->w <= 0 || rect->h <= 0) return;
	s_server_pri_recalc_list(rect);
	s_server_pri_expose(rect);
}

// ------------------------------------------------------------------
// Matrix low-level

// re-calculate rect in matrix
void s_server_pri_recalc_list(s_rect_t *rect)
{
	s_exynth_t *e = get_exynth();
	int ontop, p;

	// if the rect belongs to top window, fill at once
	int top = e->pri_top;
	if (top > 0) {
		s_rect_t *tr = &e->clients[top]->surface->buf;
		if (tr->x <= rect->x && tr->y <= rect->y &&
				tr->x + tr->w >= rect->x + rect->w &&
				tr->y + tr->h >= rect->y + rect->h) {
			s_server_surface_matrix_add(top, rect);
			return;
		}
	}

	// fill matrix with zeroes
	s_server_surface_matrix_del_coor(rect);

	// fill up the holes
	//  -1 alwaysonbottom
	//   0 regular
	//  +1 alwaysontop
	for (ontop = -1; ontop <= 1; ontop++) {
		for (p = 0; p < S_CLIENTS_MAX; p++) {
			int id = e->pri[p];
			if (id == 0) break;
			if (id >= S_CLIENTS_MAX) continue;
			if (e->clients[id] == NULL) continue;
			if (e->clients[id]->alwaysontop != ontop) continue;
			if (e->clients[id]->visible == 0) continue;

			s_server_surface_matrix_add(id, rect);
		}
	}

}

// update rect from window buffer(s) to framebuffer
void s_server_pri_expose(s_rect_t *rect)
{
	s_exynth_t *e = get_exynth();
	int i,j,cnt=0;
	unsigned char clients[S_CLIENTS_MAXB] = {0};
	debug("s_server_pri_expose(x%d y%d w%d h%d)\n",
			rect->x, rect->y, rect->w, rect->h);

	// get the list of windows in rect
	s_server_surface_matrix_find(rect, clients);
	// only one window?
	for (i=0; i<S_CLIENTS_MAXB && cnt<2; i++) {
		if (clients[i] == 0) continue;
		for (j=0; j<8 && cnt<2; j++) {
			if (clients[i] & (1<<j)) cnt++;
		}
	}
	debug("s_server_pri_expose: %d windows\n", cnt);
	if (cnt == 0) return;

	// perform server (id=0) expose
	if ((clients[0] & 1)) {
		s_server_surface_changed(rect, (cnt==1));
	}

	// perform window expose
	for (i=0; i<S_CLIENTS_MAXB; i++) {
		if (clients[i] == 0) continue;
		for (j=0; j<8; j++) {
			int id = (i<<3) | j;
			if (!(clients[i] & (1<<j))) continue;
			if (e->clients[id] == NULL) continue;
			if (e->clients[id]->visible == 0) continue;

			debug("s_server_pri_expose: sending expose to id %d\n", id);
			// if cnt==1, expose without checking matrix
			s_surface_changed(e->clients[id], rect, (cnt==1));
		}
	}
}

void s_server_pri_notify()
{
	s_exynth_t *e = get_exynth();
	int i;
	s_event_t ev;
	ev.type = MONITOR_EVENT;
	ev.data.idwnd = e->pri_top;

	for (i=1; i<S_CLIENTS_MAX; i++) {
		if (e->clients[i] == NULL) continue;
		if (e->clients[i]->type & WINDOW_MONITOR) {
			s_window_event(e->clients[i], &ev);
		}
	}
}
