#ifndef GUIIMAGE_H_
#define GUIIMAGE_H_

#include "GuiPrimitive.h"

#define GUIIMAGE(f,p) getPrimitive<GuiImage>(f,p)

/**
 * \brief Image primitive
 * 
 * Implements a static image element. Image data is loaded from
 * a specified graphical file and rendered to the containing window when
 * required. 
 * 
 * XML code below defines an image primitive:
 * 
 * \code
 *	<item id="ID" type="image" x="X" y="Y" width="W" height="H">
 * 		<init file="/path/to/image" />
 * 		<touch x="tX" y="tY" width="tW" height="tH"> />
 * 	</item>
 * \endcode
 * 
 * Where
 * - ID represents the primitive unique identifier
 * - X, Y, W, H is the set of primitive coordinates in the parent frame
 * - /path/to/file specifies the location of an image file
 * - tX, tY, tW, tH are the coordinates of the touchable area in the parent frame 
 * 
 * Touchable area is optional and usually required only for button type elements.
 * 
 * \author Andrei Birjukov
 */

class GuiImage : public GuiPrimitive
{
public:

	GuiImage(int idResource, GuiFrame* pFrame);
	virtual ~GuiImage();
	
	bool loadImage(const char *fileName);
	void close();

protected:

	virtual void onPaint(GuiSurface *drawSurface, s_rect_t &paintArea);

private:

	s_image_t*	imgData;
};

#endif /*GUIIMAGE_H_*/
