#ifndef GUIIMAGECACHE_H_
#define GUIIMAGECACHE_H_

#include <exynth/exynth.h>
#include <exynth/image.h>

#include <map>
#include <string>

/** Entry of the image cache */
struct IMAGE_CACHE_ENTRY
{
	/** Pointer to image stored in cache */
	s_image_t	*pImgData;

	/** Number of references that point to the image */
	int			nRefs;
};

/**
 * \brief Image cache implementation
 * 
 * Implements the graphical image cache. Calling the static open
 * method opens an image file. If it is open already, then the existing copy 
 * is returned and reference counter is incremented. Similarly, when the image 
 * is closed, the counter is decremented and when it reaches zero, image 
 * resources are released.  
 * 
 * \author Andrei Birjukov
 */

class GuiImageCache
{
public:
	
	static bool init();
	static bool release();
	
	static s_image_t* openImage(const char* fileName);
	static void closeImage(s_image_t* imageData);
	static unsigned int memUsage();

private:
	
	static std::map<std::string, IMAGE_CACHE_ENTRY*> imageMap;
};

#endif /*GUIIMAGECACHE_H_*/
