/***************************************************************************
 *   Copyright            : (c) 2007-2008 Artec Design LLC                 *
 *                        : (c) 2007-2008 Andrei Birjukov                  *
 *                        : (c) 2007-2008 Anti Sullin                      *
 *   Contact              : andrei.birjukov at artecdesign dot ee          *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include "GuiRectangle.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Class constructor
 *
 * Default constructor for the GUI rectangle objects. Initializes the
 * foreground color to RGB (255, 255, 255)
 *
 * \param idResource Unique primitive identifier.
 * \param pFrame Pointer to a parent frame containing the primitive.
 */
GuiRectangle::GuiRectangle(int idResource, GuiFrame* pFrame)
		: GuiPrimitive(idResource, pFrame)
{
	colForeGround.red = colForeGround.green = colForeGround.blue = 255;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Class desctructor
 *
 * Destructor for the rectangle primitive objects.
 */
GuiRectangle::~GuiRectangle()
{
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Set foreground color
 *
 * Sets foreground color of the rectangle object. Foreground color is used
 * for filling the solid rectangle.
 *
 * \param rgb Pointer to text holding the RGB color value that should be used
 */
void GuiRectangle::setForeGround(const char* rgb)
{
	colForeGround = GuiSurface::stringToRGB(rgb);
}


////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Painting function of the primitive interface
 *
 * Called by the parent frame whenever a primitive redraw is required.
 * \param drawSurface Pointer to a graphical surface where the primitive should
 * draw itself to.
 * \param paintArea The rectangular area of the parent frame surface that
 * should be re-painted by the primitive.
 */
void GuiRectangle::onPaint(GuiSurface *drawSurface, s_rect_t &paintArea)
{
	if(!isVisible) return;

	s_rect_t absRect;
	if(s_rect_intersect(&paintArea, &drawArea, &absRect)) return;

	drawSurface->fillRect(&absRect, drawSurface->makeColor(&colForeGround));
}
