/***************************************************************************
 *   Copyright            : (c) 2007-2008 Artec Design LLC                 *
 *                        : (c) 2007-2008 Andrei Birjukov                  *
 *                        : (c) 2007-2008 Anti Sullin                      *
 *   Contact              : andrei.birjukov at artecdesign dot ee          *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "GuiResource.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Class constructor
 *
 * Default constructor for the GuiResource objects. Does nothing.
 */
GuiResource::GuiResource()
{
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Class destructor
 *
 * Destructor for the GuiResource objects. Does nothing.
 */
GuiResource::~GuiResource()
{
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Add a string resource.
 *
 * Inserts a new string resource to GuiResource. If ID already
 * exists, the previous one is overwritten.
 *
 * \param id String resource identifier
 * \param val String value
 * \return Returns true if the string is added successfully.
 */
bool GuiResource::setString(int id, const char *val)
{
	stringMap[id] = val;
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Find a string resource.
 *
 * Looks up a new string resource with a given ID from GuiResource.
 * If ID already exists, the string is returned, otherwise an empty string is
 * returned.
 *
 * \param id String resource identifier
 * \return Returns resource with identifier ID or an empty string if the resource is not found.
 */
std::string GuiResource::getString(int id)
{
	GUIRESOURCE_STR_MAP::iterator it = stringMap.find(id);
	if(it != stringMap.end()) return it->second;
	else return "";
}

