#ifndef GUIRESOURCE_H_
#define GUIRESOURCE_H_

#include <map>
#include <string>

#define GUIRESOURCE_STR_MAP std::map<int, std::string>

/**
 * \brief GUI window resource
 *
 * Window resource encapsulates resources, that can be dynamically loaded from
 * template file. Currently only string resources are supported.
 *
 * To use GuiResource, create a window, that is derived from a GuiWindow or its subclass
 * and from GuiResource. Then, declare the texts as following:
 *
 * \code
 *  <window id="ID">
 * 		<!-- frames are defined here -->
 * 		<sting id="ID" value="Text"/>
 * 	</window>
 * \endcode
 *
 * \author Anti Sullin
 */

class GuiResource
{
public:
	GuiResource();
	virtual ~GuiResource();

	bool setString(int id, const char *val);
	std::string getString(int id);

private:
	GUIRESOURCE_STR_MAP stringMap;
};

#endif /* GUIRESOURCE_H_ */
