/*
 * GuiTimer.cpp
 *
 *  Created on: 12.02.2010
 *      Author: anti
 */

#include "GuiTimer.h"
#include "GuiLibrary.h"

#define DEFAULT_INTERVAL 1000

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Class constructor
 *
 * Default constructor for GuiTimer objects.
 */
GuiTimer::GuiTimer()
{
	interval = DEFAULT_INTERVAL;
	timer = NULL;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Class destructor
 *
 * Destructor for the GUI timer objects. Stops the timer, if it is running.
 */
GuiTimer::~GuiTimer()
{
	stopTimer();
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Start timer
 *
 * Starts the timer. If the timer was already started, it will be stopped and
 * then re-started with new settings.
 *
 * \return Returns true if timer was started successfully
 */
bool GuiTimer::startTimer()
{
	if (!GuiLibrary::timedlock(1000)) return false;

	if(timer) {
		s_timer_del(timer);
		s_timer_uninit(timer);
		timer = NULL;
	}

	if (s_timer_init(&timer) != 0) return false;
	timer->cb = timerProc;
	timer->data = this;
	timer->timeval = interval;
	if (s_timer_add(timer) != 0) {
		s_timer_uninit(timer);
		timer = NULL;
		GuiLibrary::unlock();
		return false;
	}
	GuiLibrary::unlock();
	return true;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Stop timer
 *
 * Stops the timer.
 */
void GuiTimer::stopTimer()
{
	if (!GuiLibrary::timedlock(1000)) return;
	if(timer) {
		s_timer_del(timer);
		s_timer_uninit(timer);
		timer = NULL;
	}
	GuiLibrary::unlock();
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Timer static call-back function
 *
 * Called by eXynth when timer has fired
 *
 * \param window eXynth window pointer
 * \param timer Pointer to timer structure that has been fired
 */
void GuiTimer::timerProc(s_window_t *window, s_timer_t *timer)
{
	GuiTimer *pOwner = (GuiTimer*)timer->data;
	if (!pOwner) return;

	pOwner->onTimer();
}
