/***************************************************************************
 *   Copyright            : (c) 2008-2009 Artec Design LLC                 *
 *                        : (c) 2008-2009 Andrei Birjukov                  *
 *                        : (c) 2008-2009 Anti Sullin                      *
 *                        : (c) 2008-2009 Jaanus Sepp                      *
 *   Contact              : andrei.birjukov at artecdesign dot ee          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "BasicReader.h"

//#define DEBUG
#include "Debug.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Reader initialization routine
 *
 * Reader implementation should override this function to implement a hardware
 * initialization procedure. Reader manager calls this function during system
 * initialization.
 *
 * \return Returns true if initialization was successful.
 */
bool BasicReader::initialize()
{
	return false;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Reader shutdown routine
 *
 * Reader implementation should override this function to implement a hardware
 * de-initialization procedure. Reader manager calls this function during
 * system shutdown.
 */
void BasicReader::shutdown()
{
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Prepare for reading
 *
 * Reader implementation should override this function to prepare the reader
 * hardware for data acquisition before an actual read occurs. This function
 * is called by the reader manager when user initiates reading.
 *
 * Read initialization is separated from the actual read to allow two-stage
 * triggering, for example to enable for reader aiming.
 *
 * A typical successful read is followed by the startRead() call and terminated
 * by the stopRead() call.
 */
void BasicReader::initRead()
{
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Start reading
 *
 * Reader implementation should override this function to start capturing data
 * from the reader hardware . This function is called by the reader manager
 * when user triggers reading.
 *
 * A typical successful read is terminated by the stopRead() call.
 */
void BasicReader::startRead()
{
}

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Stop reading
 *
 * Reader implementation should override this function to stop capturing data
 * from the reader hardware. This function is called by the reader manager when
 * user detriggers reading or a reader timeout occurs.
 */
void BasicReader::stopRead()
{
}
