#ifndef BASICREADER_H_
#define BASICREADER_H_

#include <string>

/** Reader types */
enum Reader {
	/** Barcode reader */
	READER_BARCODE,

	/** RFID reader */
	READER_RFID,

	/** UHF RFID reader */
	READER_UHF_RFID,
};

// Returned to GUI as EV_READER
/** Data returned from reader as evenet EV_READER */
struct ReaderData {
	/** Reader type the data came from */
	Reader reader;
	/** Field to store tag ID when using RFID tags. When reading barcodes, this is set to NULL **/
	const char* pId;

	/** Field to store tag data. **/
	const char* pData;

	/** Whether the data from reader is handled or not. If data is not handled, other functions can capture the data for handling. */
	int handled;
};

// Returned to GUI as EV_READER_ERR
/** Error generated by reader as event EV_READER_ERR */
struct ReaderError {
	/** Reader type the data came from */
	Reader reader;

	/** Error message retrieved from reader */
	const char* msg;

	/** Whether the error is handled or not. If not handled, other functions can capture the error for handling. */
	int handled;
};

////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Reader base class
 *
 * Implements reader basic functionality and provides interface for reader
 * initialization and control. All system readers are inheriting this class.
 *
 * Every reader must be initialized before the reader will be functional.
 * The reader can be activated before the reading. The reading process can be started
 * and stopped separately.
 */

class BasicReader
{
public:
	virtual bool initialize();
	virtual void shutdown();

	virtual void initRead();
	virtual void startRead();
	virtual void stopRead();
};

#endif /*BASICREADER_H_*/
