/*
 * BluetoothConnection.cpp
 *
 *  Created on: 21.06.2011
 *      Author: ivari
 */


#include <platform.h>

#include "BluetoothConnection.h"

//#define DEBUG
#include "Debug.h"

BluetoothConnection::BluetoothConnection(BT_CONNECTION_SETTINGS *pSettings) : NetworkConnection(COMM_MODULE_BT) {

	cfg = *pSettings;
	bluetoothModule = new BlueGigaBluetoothModule(BT_MODULE_DEVICE);
}

BluetoothConnection::~BluetoothConnection() {
	if(bluetoothModule != NULL)
		delete bluetoothModule;
}

bool BluetoothConnection::isConnectionUp() {
	return true;
}

bool BluetoothConnection::netInitialize() {
	PowerManager *pm = PowerManager::getPowerManager();
	if(pm == NULL) {
		debug("Could not get power manager\n");
		return false;
	}

	if(!pm->btPowerOn()) {
		debug("Could not power on BT module\n");
		return false;
	}

	if(!bluetoothModule->open()) {
		debug("Could not open BT module port\n");
		return false;
	}

	if(!bluetoothModule->isAlive()) {
		debug("Bluetooth module not anwsering\n");
		return false;
	}

	if(!bluetoothModule->enableMUX()) {
		debug("Could not enable MUX\n");
		return false;
	}

	if(!bluetoothModule->setLAP(cfg.lap.c_str())) {
		debug("Could not set BT LAP\n");
		return false;
	}



	bluetoothModule->setEnabled(true);
	return true;
}

bool BluetoothConnection::netShutdown() {
	PowerManager *pm = PowerManager::getPowerManager();
	if(pm == NULL) {
		debug("Could not get power manager\n");
		return false;
	}

	if(!pm->btPowerOff()) {
		debug("Error powering of BT module\n");
		return false;
	}

	bluetoothModule->setEnabled(false);
	return true;
}

bool BluetoothConnection::netConnect() {
	return true;
}

bool BluetoothConnection::netDisconnect() {
	return true;
}

bool BluetoothConnection::netSuspend() {
	return true;
}

bool BluetoothConnection::netResume() {
	return true;
}

bool BluetoothConnection::getNetworkInfo(GENERIC_NETWORK_INFO &netInfo) {
	netInfo.isConnected = isConnectionUp();
	netInfo.signalStrength = bluetoothModule->getSignalStrength();
	netInfo.networkAddress = "n/a";
	netInfo.networkName = "BT";
	netInfo.networkChannel = 0;
	netInfo.hardwareAddress = "n/a";

	debug("%d\n", netInfo.signalStrength);
	return true;
}



