/*
 * BluetoothSocket.cpp
 *
 *  Created on: 29.06.2011
 *      Author: ivari
 */

#include "BluetoothSocket.h"

//#define DEBUG
#include "Debug.h"

BluetoothSocket::BluetoothSocket(BluetoothConnection *btConn) {
	bt = btConn;
	spp = NULL;

	keepAliveCount = 0;
}

BluetoothSocket::~BluetoothSocket() {
}


bool BluetoothSocket::connect(const unsigned char *dstAddr, int dstPort, const char *pin) {
	if(bt == NULL) return false;

	if(spp==NULL) {
		debug("Creating new SPP connection addr: %s, port: %d, pin: %s\n", dstAddr, dstPort, pin);
		spp = bt->getBluetoothModule()->createSPPConnection(dstAddr, dstPort, pin);
		if(spp == NULL) return false;
	}

	return spp->connect();
}

bool BluetoothSocket::connect(const char* host, int port) {
	return connect((unsigned char *)host, 0, "");
}

bool BluetoothSocket::close() {
	if(spp != NULL) {
		debug("Closing SPP connection\n");

		bt->getBluetoothModule()->deleteSPPConnection(spp);
		spp = NULL;
		return true;
	}

	return true;
}

size_t BluetoothSocket::readData(unsigned char* pData, size_t maxLen) {
	if(spp == NULL || !spp->isConnected()) return 0;
	if(!spp->isInputPending(1000)) return 0;

	return spp->readData(pData, maxLen);
}

bool BluetoothSocket::writeData(unsigned char *pData, size_t dataLen) {
	if(spp == NULL || !spp->isConnected()) return false;

	return spp->writeData(pData, dataLen)>0;
}


size_t BluetoothSocket::getPendingInput() {
	return 0;
}

bool BluetoothSocket::setTimeouts(int sendTout, int recvTout) {
	return true;
}

bool BluetoothSocket::setKeepalive(int keepIdle, int keepInterval, int keepCount) {
	keepAliveCount = keepCount;

	return true;
}

bool BluetoothSocket::isConnected() {
	return spp!= NULL && spp->isConnected();
}

