/**
 *
 * Provides the following facilities for debug trace output. When the preprocessor
 * DEBUG defines are disabled, this header generates a dummy code that is optimized out.
 *
 * - debug() -- a printf-style debug trace function, requires defining DEBUG
 * - debug_dump() -- buffer dump helper, requires defining DEBUG_DUMP and
 * DEBUG_DUMP_HEX or DEBUG_DUMP_ASCII
 */

#if defined(DEBUG) || defined(DEBUG_DUMP_HEX) || defined(DEBUG_DUMP_ASCII)
#include <stdio.h>
#endif

#ifdef DEBUG
	#define debug(fmt, args...)	do {\
			printf("(%s:%d) %s: ", __FILE__, __LINE__, __FUNCTION__); \
			printf(fmt, ## args); \
			} while(0);
#else
	/** Debug function that will be used when debugging is disabled. Inserts dummy code. */
	#define debug(fmt...)
#endif

#if defined(DEBUG_DUMP_HEX) || defined(DEBUG_DUMP_ASCII)
void debug_dump(unsigned char* buf, size_t len)
{
#ifdef DEBUG_DUMP_HEX
	printf("[");
	for(size_t i=0; i<len; i++)
	{
		printf("%02x ", buf[i]);
	}
	printf("]\n");
#endif
#ifdef DEBUG_DUMP_ASCII
	printf("[");
	for (size_t i = 0; i < len; i ++)
	{
		if (buf[i] > 31 && buf[i] < 127)
			printf("%c", buf[i]);
		else printf(".");
	}
	printf("]\n");
#endif
}
#else
	/** Debug dump function that will be used when debugging is disabled. Inserts dummy code. */
	#define debug_dump(...)
#endif
