/*
 * GprsPpp.h
 *
 *  Created on: Sep 26, 2008
 *      Author: andrei
 */

#ifndef GPRSPPP_H_
#define GPRSPPP_H_

/** PID file for PPP daemon */
#define DAEMON_PID_FILE			"/var/run/ppp0.pid"

/** PPP daemon executable location */
#define DAEMON_EXECUTABLE		"/usr/sbin/pppd"

/** PPP interface name */
#define DAEMON_INTERFACE		"ppp0"

/** Name of the GPRS profile */
#define GPRS_PROFILE_NAME		"gprs-data"

/** Timeout for starting PPP daemon */
#define DAEMON_START_TOUT		300

/** Timeout for stopping PPP daemon */
#define DAEMON_STOP_TOUT		100

/** Structure to hold GPRS PPP parameters */
struct GPRSPPP_PARAMETERS
{
	/** Enable authentication */
	bool		useAuth;
	/** User name for GPRS PPP authentication */
	const char*	gprsUser;
	/** Password for GPRS PPP authentication */
	const char*	gprsPass;
	/** Enable debugging, passed further to PPP daemon */
	bool		debugMode;
	/** Persistent mode, PPP mode does not close on error
	 * and daemon attempts to recover */
	bool		persistMode;
};

/**
 * \brief GPRS PPP control class
 *
 * Implements a number of static functions to simplify management of the
 * GPRS PPP daemon. The GPRS PPP daemon (pppd) must be bound to a certain
 * serial port, either physical or virtual. The GPRS modem serial port
 * must be used as the physical port. Alternatively, GSM 07.10 multiplexer
 * may provide a virtual port.
 */
class GprsPpp
{
public:

	static bool start(const char *serialPort, GPRSPPP_PARAMETERS *pParams);
	static bool stop();

	static bool isRunning();

private:

	static int getDaemonPID();
};

/** \file */
#endif /* GPRSPPP_H_ */
