#ifndef MODEMPORT_H_
#define MODEMPORT_H_

#include "SerialPort.h"

/** Default timeout for AT commands */
#define DEFAULT_AT_TIMEOUT			5000

/** Valid AT command replies of the GSM/GPRS modem. */
enum ModemAnswerType
{
	/** Invalid reply, error condition */
	AT_INVALID = -1,
	/** OK reply, success confirmation */
	AT_OK = 0,
	/** Connect confirmation */
	AT_CONNECT,
	/** RING signal */
	AT_RING,
	/** No carrier reply */
	AT_NOCARIER,
	/** Generic error reply */
	AT_ERROR,
	/** No answer reply */
	AT_NOANSWER,
	/** Specific error information */
	AT_CMS_ERROR,
	/** Specific error information */
	AT_CME_ERROR
};

/**
 * \brief Modem port class
 *
 * Modem port class implements parsing of modem AT commands and responses.
 * The class is intended to simplify communication to a GSM/GPRS modem with
 * generic AT command interface.
 */
class ModemPort: public SerialPort
{
public:

	ModemPort();
	ModemPort(const char* portName);
	virtual ~ModemPort();

	size_t read(char *pData, size_t maxSize, int timeOut = DEFAULT_AT_TIMEOUT);

	void waitNoCarrier(void);
	ssize_t write(const char *pData);

	/**
	 * \brief Get the modem answer as answer type
	 * \return Returns the answer type retrieved from modem
	 */
	ModemAnswerType getAnswer()
	{
		return answerType;
	}

private:

	bool checkNoCarrier(char inChar);

	ModemAnswerType answerType;

	static const char* modemAnswerStrings[];
	static size_t modemAnswerSizes[];
	size_t noCarrierDetect;
};

/** \file */
#endif /*MODEMPORT_H_*/
