/*
 * NetworkEventListener.h
 *
 *  Created on: Nov 19, 2008
 *      Author: andrei
 */

#ifndef NETWORKEVENTLISTENER_H_
#define NETWORKEVENTLISTENER_H_

/** Events generated by a network connection. */
enum NetworkEvent
{
	/** Offline event */
	NE_OFFLINE,
	/** Connecting in progress event */
	NE_CONNECTING,
	/** Connected state event */
	NE_CONNECTED,
};

/**
 * \brief Class to retrieve events from network devices
 * Retrieves notifications from network devices regarding the network events.
 *
 */
class NetworkEventListener
{
public:
	/**
	 * \brief Network event report
	 *
	 * Called by the network connection to notify the listener about a new event.
	 *
	 * \param e Contains the network event type identifier.
	 */
	virtual void networkEvent(NetworkEvent e) = 0;

	/**
	 * \brief Network signal quality report
	 *
	 * Called by the network connection periodically to notify the listener about
	 * the network received signal quality.
	 *
	 * \param signalLevel Specifies the signal level, in percent 0..100.
	 */
	virtual void networkSignalQuality(int signalLevel) = 0;

	/**
	 * \brief Network error report
	 *
	 * Called by the network connection to notify the listener about a network error.
	 *
	 * \param error Contains the network error description.
	 */
	virtual void networkError(const char *error) = 0;

	/**
	 * \brief Network status change event
	 *
	 * Called by the network connection to notify the listener of a status change.
	 *
	 * \param error Contains the network new status.
	 */
	virtual void networkStatus(const char *error) = 0;
};

#endif /* NETWORKEVENTLISTENER_H_ */
