#ifndef STDIOFILE_H_
#define STDIOFILE_H_

#include <stdio.h>
#include <stdint.h>
#include <string>

#include <stdio.h>

/**
 * \brief Standard I/O file class
 *
 * I/O file class is a wrapper for basic libC stream I/O calls. StdioFile class also calls
 * Media class methods internally, to support for transparent media mounting.
 */
class StdioFile
{
public:

	/** File access modes */
	enum OpenMode
	{
		/** Read only access. Opening fails if the file does not exist. */
		ModeRead,
		/** Write only access. The file is truncated if it exists already,
		 * otherwise a new file is created. */
		ModeWrite,
		/** Read and write access. The file is created if it does not exist,
		 * otherwise it is truncated. */
		ModeReadWrite,
		/** Read and append access. The file is created if it does not exist. */
		ModeAppend,
		/** Open a file for update both reading and writing. The file must exist. */
		ModeUpdate
	};

	StdioFile();
	virtual ~StdioFile();

	bool open(const char* fileName, OpenMode mode);
	bool close();
	bool isEOF();

	bool writeData(const void* pData, size_t dataLen);
	size_t readData(void* pData, size_t maxLen);

	long getSize();

	bool writeString(const char* strData);
	std::string readString();

protected:
	/** File handle for currently opened file */
	FILE*	fileStream;

private:

	/** Mount point associated with the mounted media. Should be used for unmounting. */
	int		mountPoint;
};

#endif /*STDIOFILE_H_*/
