/*
 * TcpSocket.h
 *
 *  Created on: Sep 29, 2008
 *      Author: andrei
 */

#ifndef TCPSOCKET_H_
#define TCPSOCKET_H_

#define DEFAULT_SEND_TIMEOUT	30000
#define DEFAULT_RECV_TIMEOUT	30000

#include <stdlib.h>

#include "Socket.h"

/**
 * \brief TCP socket class
 *
 * Provides a wrapper class implementation for the BSD compatible socket I/O.
 * Socket class is designed to simplify the handling of a blocking client socket
 * in the user application.
 */
class TcpSocket : public Socket
{
public:

	TcpSocket();
	virtual ~TcpSocket();

	bool connect(const char* host, int port);
	bool close();

	bool isConnected();

	bool writeData(unsigned char *pData, size_t dataLen);
	size_t readData(unsigned char* pData, size_t maxLen);

	size_t getPendingInput();
	size_t getPendingOutput();

	bool setTimeouts(int sendTout, int recvTout);
	bool setKeepalive(int keepIdle, int keepInterval, int keepCount);

protected:

	/** Handle to the TCP socket opened */
	int socketHandle;
	/** Receive timeout setting */
	int recvTimeout;
	/** Send timeout setting */
	int sendTimeout;
};

#endif /* TCPSOCKET_H_ */
