/*
 * lcdecho.c
 *
 *..Modified on: 29.09.2015
 *      Modified by: Silver Kits <silver.kits@vecly.com>
 */

#include <stdarg.h>
#include "fbutils.h"
#include <string.h>
#include <stdio.h>

void printRow(int row, const char *buf, int transp)
{
	int y;

	y = row * 22;
	if (!transp)
		fillrect(0, y, xres, y+22, 0);
	put_string(5, y, buf, 1);
}


/* USAGE:
 * echo -b 0x000000 -f 0xFFFFFF 1 "Text"
 *     echoes white Text to line 1
 * echo 1
 *     echoes standard input to line 1
 * arguments:
 *  -b  background colour
 *  -f  foreground colour
 *  -t  transparent
 * */
int main(int argc, char *argv[])
{
	int lineid = 0, i;
	char buf[64];
	int cnt=0;
	int start=0;
	int fcol = 0xFFFFFF;
	int bcol = 0x000000;
	int multiline=1;
	int transparent = 0;
	int tmp;
	argc--;

	for (i=1; i<=argc; i++) {
		if (argv[i][0]=='-') {
			if (strcmp(argv[i], "-f")==0) {
				if (i+1<= argc && sscanf(argv[i+1], "0x%6x", &fcol)!=1) {
					printf ("Syntax error: should be -f 0x111111");
					return -2;
				}
				i++;
			} else if (strcmp(argv[i], "-b")==0) {
				if (i+1<= argc && sscanf(argv[i+1], "0x%6x", &bcol)!=1) {
					printf ("Syntax error: should be -b 0x111111");
					return -2;
				}
				i++;
			} else if (strcmp(argv[i], "-n")==0) {
				multiline = 0;
			} else if (strcmp(argv[i], "-t")==0) {
				transparent = 1;
			}
		} else
		// we have a non-option argument
		if (!start)
			start=i;
	}

	if (!start) {
		printf("USAGE:\necho -b 0x000000 -f 0xFFFFFF 1 \"Text\"\n    echoes white Text to line 1\n\necho 1\n    echoes standard input to line 1\n\narguments:\n -b  background colour\n -f  foreground colour\n -t  transparent\n\n");
		return -1;
	}
	open_framebuffer();
	setcolor(0, bcol);
	setcolor(1, fcol);

	sscanf(argv[start], "%d", &lineid);

	if (argc == start) {
		// the only argument was line number
		tmp = getchar();
		while (tmp != EOF && ++cnt < (sizeof(buf) - 1)){
			buf[cnt] = tmp & 0xFF;
			if (multiline && buf[cnt-1] == '\n') {
				buf[cnt-1] = 0;
				printRow(lineid, buf, transparent);
				cnt=0;
				lineid++;
			}
			tmp = getchar();
		}
		buf[cnt]=0;
		printRow(lineid, buf, transparent);
	}
	else
	{
		// one more argument - the text
		printRow(lineid, argv[start+1], transparent);
	}
	close_framebuffer();
	return 0;
}

