/*
 * linux/arch/arm/mach-at91/board-spatha.c
 *
 *  Copyright (C) 2005 SAN People
 *  Copyright (C) 2006 Atmel
 *  Copyright (C) 2008 Artec Design <anti.sullin@artecdesign.ee>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/types.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/mtd/physmap.h>
#include <linux/spi/spi.h>
#include <linux/input.h>
#include <linux/gpio_keys.h>
#include <linux/magstripe_magtek.h>
#include <linux/pwrctrl.h>
#include <linux/rfid_nxp.h>
#include <linux/rfid_ams.h>

#include <mach/hardware.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/irq.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/board.h>
#include <mach/gpio.h>
#include <linux/wdt.h>
#include <mach/at91_shdwc.h>

#include <linux/fb.h>
#include <video/atmel_lcdc.h>
#include <linux/spi/ads7846.h>
#include <linux/i2c-gpio.h>
#include <linux/gpio_keyb.h>
#include <linux/serial_8250.h>
#include <linux/i2c/at24.h>

#include "generic.h"

#define OLED_C0283QGLC

/*
 * Additional serial port
 */
#if defined(CONFIG_SERIAL_8250) || defined(CONFIG_SERIAL_8250_MODULE)

static struct plat_serial8250_port serial_8250_data[] = {
	{
		.mapbase	= (unsigned long)(AT91_CHIPSELECT_2),
		.irq		= AT91_PIN_PC11,
		.uartclk	= 14745600,
		.iotype		= UPIO_MEM,
		.flags		= UPF_BOOT_AUTOCONF|UPF_IOREMAP,
		.regshift	= 0,
	},
	{ },
};


static struct platform_device serial_8250_device = {
	.name		= "serial8250",
	.id		= PLAT8250_DEV_PLATFORM,
	.dev		= {
		.platform_data	= serial_8250_data,
	}
};

static void __init add_device_serial_8250(void)
{
	platform_device_register(&serial_8250_device);
}
#else
static void __init add_device_serial_8250(void) {}
#endif



/*
 * Generic system initializations
 */

static void __init spatha_map_io(void)
{
	/* Initialize processor: 13.560 MHz crystal */
	at91sam9261_initialize(13560000);

	/* DGBU on ttyAT0. (Rx & Tx only) */
	at91_register_uart(0, 0, 0);

	/* GPRS on ttyAT1. (all pins) */
	at91_register_uart(AT91SAM9261_ID_US1, 1, ATMEL_UART_RTS | ATMEL_UART_CTS);
	/* 2D on ttyAT2. (all pins) */
	at91_register_uart(AT91SAM9261_ID_US2, 2, ATMEL_UART_RTS | ATMEL_UART_CTS);

	/* set serial console to DBGU */
	// We do not want this default. If needed, the console is set
	// via console= kernel command line option.
	//at91_set_serial_console(0);
}

static void __init spatha_init_irq(void)
{
	at91sam9261_init_interrupts(NULL);
}

/*
 * NOR flash
 */

static struct physmap_flash_data spatha_flash_data = {
	.width		= 2,
};

static struct resource spatha_flash_resource = {
	.start		= AT91_CHIPSELECT_0,
	.end		= AT91_CHIPSELECT_0 + SZ_16M - 1,
	.flags		= IORESOURCE_MEM,
};

static struct platform_device spatha_flash = {
	.name		= "physmap-flash",
	.id		= 0,
	.dev		= {
		.platform_data	= &spatha_flash_data,
	},
	.num_resources	= 1,
	.resource	= &spatha_flash_resource,
};

static void __init add_device_flash(void)
{
	platform_device_register(&spatha_flash);
}

/*
 * USB Host Port
 */
static struct at91_usbh_data __initdata spatha_usbh_data = {
	.ports		= 2,
};

/*
 * USB Device Port
 */
static struct at91_udc_data __initdata spatha_udc_data = {
	.vbus_pin	= AT91_PIN_PB21,
	.pullup_pin	= 0,		/* pull-up driven by UDC */
};

/*
 * MCI (SD/MMC)
 */
static struct at91_mmc_data __initdata spatha_mmc_data = {
	.wire4		= 1,
	.vcc_pin	= AT91_PIN_PB24,
	.vcc_inv	= 1,
	.mcda0_pin	= AT91_PIN_PA0,
};



/*
 * ADS7846 Touchscreen (optional, NL)
 */
#if defined(CONFIG_TOUCHSCREEN_ADS7846) || defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE)

#define	MAX_12BIT	((1<<12)-1)
#define X_EDGE_PERCENT 7
#define Y_EDGE_TOP_PERCENT 8
#define Y_EDGE_BTM_PERCENT 7
#define X_EDGE (MAX_12BIT * X_EDGE_PERCENT / 100)
#define Y_EDGE_TOP (MAX_12BIT * Y_EDGE_TOP_PERCENT / 100)
#define Y_EDGE_BTM (MAX_12BIT * Y_EDGE_BTM_PERCENT / 100)

static int ads7846_pendown_state(void)
{
	return !at91_get_gpio_value(AT91_PIN_PB23);	/* Touchscreen PENIRQ */
}

static struct ads7846_platform_data ads_info = {
	.x_min = X_EDGE,
	.x_max = MAX_12BIT - X_EDGE,
	.y_min = Y_EDGE_TOP,
	.y_max = MAX_12BIT - Y_EDGE_BTM,
	.x_plate_ohms	= 10,
	.pressure_max	= 14,
	.settle_delay_usecs = 1,
	.debounce_max	= 3,
	.debounce_tol	= 100,
	.vref_delay_usecs = 5000,
	.penirq_recheck_delay_usecs = 1000,
	.get_pendown_state	= ads7846_pendown_state,
};

static void __init spatha_add_device_ts(void)
{
	at91_set_gpio_input(AT91_PIN_PB25, 1);	/* Touchscreen BUSY signal */
}
#else
static void __init spatha_add_device_ts(void) {}
#endif


/*
 * SPI devices
 */

#if defined(CONFIG_SPI_ATMEL) || defined(CONFIG_SPI_ATMEL_MODULE)

// We can not have two static declarations with same CS as one of them
// will get filtered out. So we load the conflicting drivers
// dynamically after we have determined which to load.
#if defined(CONFIG_BOARD_RFID_NXP) || defined(CONFIG_BOARD_RFID_NXP_MODULE)
  static struct rfid_nxp_plat_data rfid_nxp_platdata = {
	  .pin_assignment	= { AT91_PIN_PB26,       // powerdown pin
		                    AT91_PIN_PC2,        // interrupt pin (IRQ0)
    },
    .irq = AT91SAM9261_ID_IRQ0,
  };
#endif

#if defined(CONFIG_BOARD_RFID_AMS) || defined(CONFIG_BOARD_RFID_AMS_MODULE)
  static struct rfid_ams_plat_data rfid_ams_platdata = {
	  .pin_assignment	= {		AT91_PIN_PB26,	// enable pin
								AT91_PIN_PC17,	// power pin(inverted)
    },
    .irq = AT91SAM9261_ID_IRQ0,
  };
#endif

#if defined(CONFIG_BOARD_RFID_AMS) || defined(CONFIG_BOARD_RFID_AMS_MODULE)
static struct spi_board_info spatha_ams_spi_device =
	{	// Austria Microsystems RFID chip
		.modalias	= "rfid_ams",
		.chip_select	= 1,
		.bus_num	= 1,
		.controller_data = (void *)AT91_PIN_PB27,
		.max_speed_hz	= 1600000,
		.platform_data = &rfid_ams_platdata,
	};
#endif

#if defined(CONFIG_BOARD_RFID_NXP) || defined(CONFIG_BOARD_RFID_NXP_MODULE)
static struct spi_board_info spatha_nxp_spi_device =
	{	// NXP RFID chip
		.modalias	= "rfid_nxp",
		.chip_select	= 1,
		.bus_num	= 1,
		.controller_data = (void *)AT91_PIN_PB27,
		.max_speed_hz	= 3000000,
		.platform_data = &rfid_nxp_platdata,
	};
#endif

static struct spi_board_info spatha_spi_devices[] = {

#if defined(CONFIG_TOUCHSCREEN_ADS7846) || defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE)
	{ // ADS7843 touchscreen (optional, NL)
		.modalias	= "ads7846",
		.chip_select	= 0,
		.max_speed_hz	= 1000000, //125000 * 26,	/* (max sample rate @ 3V) * (cmd + data + overhead) */
		.bus_num	= 1,
		.platform_data	= &ads_info,
		.irq		= AT91_PIN_PB23,
		.controller_data = (void *) AT91_PIN_PB28,
	},
#endif

#if defined(CONFIG_BOARD_OLED_C0283QGLC)
	{	/* CMEL C0283QGLC OLED panel SPI configuration interface */
		.modalias	= "c0283qglc_oled",
		.chip_select	= 2,
		.controller_data = (void *)AT91_PIN_PA25,
		.max_speed_hz	= 2 * 1000 * 1000,
		.bus_num	= 1,
		.platform_data = (void *)AT91_PIN_PA28,
	},
#endif

#if defined(CONFIG_BOARD_ADC081S021)
	{	// vbat ADC
		.modalias	= "adc081s021",
		.chip_select	= 3,
		.bus_num	= 1,
		.controller_data = (void *)AT91_PIN_PA26,
		.max_speed_hz	= 1000000,
		.platform_data = (void *)AT91_PIN_PB4,
	},
#endif

};
#endif


#ifdef CONFIG_BOARD_PWRCTRL
static struct pwrctrl_platform_data spatha_pwrctrl_platform_data= {
	.input_pins = {
		// System generic
		{ CTRL_GPIO, AT91_PIN_PB21 },	// I_PIN_USB_VBUS
		{ CTRL_GPIO, AT91_PIN_PA14 },	// I_PIN_LED_R
		{ CTRL_GPIO, AT91_PIN_PA24 },	// I_PIN_LED_G

		// GPRS
		{ CTRL_GPIO, AT91_PIN_PA19 },	// I_PIN_GPRS_PWRGD,

		// 1D
		{ CTRL_GPIO, AT91_PIN_PA31 },	// I_PIN_1D_PWR_DWN,

		// keypad buttons
		{ CTRL_GPIO, AT91_PIN_PA27 },	// I_PIN_BTN_POWER,
	},
	.output_pins = {
		// System generic
		{ CTRL_GPIO_OD, AT91_PIN_PA14, 1 },	// O_PIN_LED_R
		{ CTRL_GPIO_OD, AT91_PIN_PA24, 1 },	// O_PIN_LED_G

		{ CTRL_GPIO, AT91_PIN_PC16, 1 }, 	// O_PIN_EXT3VEN_N
		{ CTRL_GPIO, AT91_PIN_PC17, 0 },	// O_PIN_EXT5VEN_N
		{ CTRL_GPIO_OD, AT91_PIN_PA29, 0 },	// O_PIN_MPCIE3VEN_N

		// GPRS
		{ CTRL_GPIO_OD, AT91_PIN_PC18, 1 }, //O_PIN_GPRS_PWR : default ON
		{ CTRL_GPIO, AT91_PIN_PA18, 0 }, //O_PIN_GPRS_RST
		{ CTRL_GPIO, AT91_PIN_PA17, 0 }, //O_PIN_GPRS_ON
		{ CTRL_GPIO, AT91_PIN_PA11, 0 }, //O_PIN_GPRS_DTR

		// 1D
		{ CTRL_GPIO, AT91_PIN_PA23, 0 }, //O_PIN_1D_AIM


	},
	.kbbl_en_pin = AT91_PIN_PA3,
};

static struct resource pwrctrl_resources[] = {
	[0] = { // Backlight PWM timer base address
		.start	= AT91SAM9261_BASE_TC0,
		.end	= AT91SAM9261_BASE_TC0 + 0x40 - 1,
		.flags	= IORESOURCE_MEM,
	},
};
static struct platform_device pwrctrl_device = {
	.name		= "pwrctrl",
	.id		= -1,
	.dev		= {
				.platform_data		= &spatha_pwrctrl_platform_data,
	},
	.resource	= pwrctrl_resources,
	.num_resources	= ARRAY_SIZE(pwrctrl_resources),
};

void __init add_device_pwrctrl(void)
{
	at91_set_B_periph(AT91_PIN_PC19, 0);		/* LCD BL */
	platform_device_register(&pwrctrl_device);
}
#else
void __init add_device_pwrctrl(void) {}
#endif


#ifdef CONFIG_BOARD_WDT

struct wdt_platform_data spatha_wdt_platform_data = {
	.rst_keys = NULL,
	.num_rst_keys = 0,
};

static struct platform_device wdt_device = {
	.name		= "wdt",
	.id			= -1,
	.dev		= {
		.platform_data	= &spatha_wdt_platform_data,
	},
};

void __init add_device_wdt(void)
{
	platform_device_register(&wdt_device);
}
#else
void __init add_device_wdt(void) {}
#endif


#ifdef CONFIG_BOARD_SPKR
static struct resource at91_spkr_resources[] = {
	[0] = {
		.start	= AT91SAM9261_BASE_TC1,
		.end	= AT91SAM9261_BASE_TC1 + 0x40 - 1,
		.flags	= IORESOURCE_MEM,
	},
};
static struct platform_device at91_spkr_device = {
	.name		= "at91-spkr",
	.id		= -1,
	.dev		= {
				.platform_data		= (void *)AT91_PIN_PC20,
	},
	.resource	= at91_spkr_resources,
	.num_resources	= ARRAY_SIZE(at91_spkr_resources),
};

void __init add_device_at91_spkr(void)
{
	at91_set_B_periph(AT91_PIN_PC21, 0);		/* BEEP PWM out */
	platform_device_register(&at91_spkr_device);
}
#else
void __init add_device_at91_spkr(void) {}
#endif


#ifdef CONFIG_BOARD_MAGSTRIPE_MAGTEK
static struct resource at91_mag_resources[] = {
	[0] = {
		.start	= AT91SAM9261_BASE_SSC1,
		.end	= AT91SAM9261_BASE_SSC1 + 0x4000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = { // SSC
		.start	= AT91SAM9261_ID_SSC1,
		.end	= AT91SAM9261_ID_SSC1,
		.flags	= IORESOURCE_IRQ,
	},
	[2] = { // magsripe irq
		.start	= AT91_PIN_PA20,
		.end	= AT91_PIN_PA20,
		.flags	= IORESOURCE_IRQ,
	},
};
static struct magstripe_magtek_platform_data mag_pwrctrl_platform_data= {
	.data_pin = AT91_PIN_PA20,
	.power_pin = AT91_PIN_PA22,
};

static struct platform_device at91_mag_device = {
	.name		= "magstripe_magtek",
	.id		= -1,
	.dev		= {
				.platform_data		= &mag_pwrctrl_platform_data,
	},
	.resource	= at91_mag_resources,
	.num_resources	= ARRAY_SIZE(at91_mag_resources),
};

void __init add_device_at91_mag(void)
{
	at91_set_B_periph(AT91_PIN_PA20, 0);		/* Data */
	at91_set_B_periph(AT91_PIN_PA21, 0);		/* strobe */

	platform_device_register(&at91_mag_device);
}
#else
void __init add_device_at91_mag(void) {}
#endif


#ifdef OLED_C0283QGLC
// CMEL C0283QGLC OLED display
static struct fb_videomode toppoly_modes[] = {
	{
		.name		= "320x240 @ 75",
		.refresh	= 75,
		.xres		= 240,		.yres		= 320,
		.pixclock	= KHZ2PICOS(10000),

		.left_margin	= 4,		.right_margin	= 8,
		.upper_margin	= 8,		.lower_margin	= 8,
		.hsync_len	= 4,		.vsync_len	= 4,

		.sync		= FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
		.vmode		= FB_VMODE_NONINTERLACED,
	},
};

static struct fb_monspecs spatha_default_monspecs = {
	.modedb		= toppoly_modes,
	.manufacturer	= "CML",
	.monitor	= "C0283",
	.modedb_len	= ARRAY_SIZE(toppoly_modes),
	.hfmin		= 15000,
	.hfmax		= 64000,
	.vfmin		= 50,
	.vfmax		= 150,
	.dclkmax		= 30000000,
};
#endif

struct atmel_lcdfb_info __initdata spatha_lcdc_data = {
	.default_bpp		= 16,
	.default_dmacon		= ATMEL_LCDC_DMAEN,
	.default_lcdcon2	= (ATMEL_LCDC_DISTYPE_TFT
				   | ATMEL_LCDC_CLKMOD_ALWAYSACTIVE
				   | ATMEL_LCDC_MEMOR_LITTLE),
	.default_monspecs	= &spatha_default_monspecs,
	.guard_time		= 1,
};



/*
 * GPIO Buttons
 */
#if defined(CONFIG_BOARD_GPIO_KEYB) || defined(CONFIG_BOARD_GPIO_KEYB_MODULE)

#ifdef CONFIG_MACH_SPATHA_REV1
// --------------------------- REV1
struct gpio_keyb_direct_button spatha_direct_buttons[] = {
	{
		.keycode = KEY_POWER,
		.gpio_input = AT91_PIN_PA27,
		.active_low_input = 0,
		.desc = "POWER",
	},
};

struct gpio_keyb_matrix_button spatha_row1_buttons[] = {
	{
		.keycode = KEY_F1,
		.desc = "F1",
	},

	{
		.keycode = KEY_ENTER,
		.desc = "ENTER",
	},

	{
		.keycode = KEY_F11,
		.desc = "SHIFT",
	},

	{
		.keycode = KEY_F5,
		.desc = "F5",
	},
};

struct gpio_keyb_matrix_button spatha_row2_buttons[] = {
	{
		.keycode = KEY_F2,
		.desc = "F2",
	},

	{
		.keycode = KEY_UP,
		.desc = "UP",
	},

	{
		.keycode = KEY_DOWN,
		.desc = "DOWN",
	},

	{
		.keycode = KEY_F4,
		.desc = "F4",
	},
};

struct gpio_keyb_matrix_button spatha_row3_buttons[] = {
	{
		.keycode = KEY_F3,
		.desc = "F3",
	},

	{
		.keycode = KEY_7,
		.desc = "7",
	},

	{
		.keycode = KEY_8,
		.desc = "8",
	},

	{
		.keycode = KEY_9,
		.desc = "9",
	},
};

struct gpio_keyb_matrix_button spatha_row4_buttons[] = {
	{
		.keycode = KEY_4,
		.desc = "4",
	},

	{
		.keycode = KEY_5,
		.desc = "5",
	},

	{
		.keycode = KEY_6,
		.desc = "6",
	},

	{
		.keycode = KEY_3,
		.desc = "3",
	},
};

struct gpio_keyb_matrix_button spatha_row5_buttons[] = {
	{
		.keycode = KEY_1,
		.desc = "1",
	},

	{
		.keycode = KEY_2,
		.desc = "2",
	},

	{
		.keycode = KEY_0,
		.desc = "0",
	},

	{
		.keycode = KEY_MINUS,
		.desc = "MINUS",
	},
};

struct gpio_keyb_matrix_button spatha_row6_buttons[] = {
	{
		.keycode = KEY_DOT,
		.desc = "DOT",
	},

	{
		.keycode = KEY_BACKSPACE,
		.desc = "BACKSPACE",
	},

	{
		.keycode = KEY_F12,   // unused by hardware
		.desc = "F12",
	},

	{
		.keycode = KEY_F13,
		.desc = "F13",      // unused by hardware
	},
};


struct gpio_keyb_matrix_row spatha_matrix_rows[] = {
	{
		.buttons = spatha_row1_buttons,
		.gpio_output = AT91_PIN_PC30,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row2_buttons,
		.gpio_output = AT91_PIN_PC29,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row3_buttons,
		.gpio_output = AT91_PIN_PC28,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row4_buttons,
		.gpio_output = AT91_PIN_PC26,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row5_buttons,
		.gpio_output = AT91_PIN_PC23,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row6_buttons,
		.gpio_output = AT91_PIN_PC22,
		.active_low_output = 1,
	},

};

struct gpio_keyb_matrix_col spatha_matrix_cols[] = {
	{
		.gpio_input = AT91_PIN_PC31,
		.active_low_input = 1,
	},

	{
		.gpio_input = AT91_PIN_PC27,
		.active_low_input = 1,
	},

	{
		.gpio_input = AT91_PIN_PC25,
		.active_low_input = 1,
	},

	{
		.gpio_input = AT91_PIN_PC24,
		.active_low_input = 1,
	},
};
// --------------------------- END OF REV1
#else
// --------------------------- REV2
struct gpio_keyb_direct_button spatha_direct_buttons[] = {
	{
		.keycode = KEY_POWER,
		.gpio_input = AT91_PIN_PA27,
		.active_low_input = 0,
		.desc = "POWER",
	},
};

struct gpio_keyb_matrix_button spatha_row1_buttons[] = {
	{
		.keycode = KEY_F1,
		.desc = "F1",
	},

	{
		.keycode = KEY_ENTER,
		.desc = "ENTER",
	},

	{
		.keycode = KEY_F4,
		.desc = "F4",
	},

	{
		.keycode = KEY_F5,
		.desc = "F5",
	},
};

struct gpio_keyb_matrix_button spatha_row2_buttons[] = {
	{
		.keycode = KEY_F2,
		.desc = "F2",
	},

	{
		.keycode = KEY_UP,
		.desc = "UP",
	},

	{
		.keycode = KEY_DOWN,
		.desc = "DOWN",
	},

	{
		.keycode = KEY_BACKSPACE,
		.desc = "BACKSPACE",
	},
};

struct gpio_keyb_matrix_button spatha_row3_buttons[] = {
	{
		.keycode = KEY_F3,
		.desc = "F3",
	},

	{
		.keycode = KEY_1,
		.desc = "1",
	},

	{
		.keycode = KEY_2,
		.desc = "2",
	},

	{
		.keycode = KEY_3,
		.desc = "3",
	},
};

struct gpio_keyb_matrix_button spatha_row4_buttons[] = {
	{
		.keycode = KEY_4,
		.desc = "4",
	},

	{
		.keycode = KEY_5,
		.desc = "5",
	},

	{
		.keycode = KEY_6,
		.desc = "6",
	},

	{
		.keycode = KEY_9,
		.desc = "9",
	},
};

struct gpio_keyb_matrix_button spatha_row5_buttons[] = {
	{
		.keycode = KEY_7,
		.desc = "7",
	},

	{
		.keycode = KEY_8,
		.desc = "8",
	},

	{
		.keycode = KEY_0,
		.desc = "0",
	},

	{
		.keycode = KEY_F11,
		.desc = "F11",
	},
};

struct gpio_keyb_matrix_button spatha_row6_buttons[] = {
	{
		.keycode = KEY_DOT,
		.desc = "DOT",
	},

	{
		.keycode = KEY_MENU,
		.desc = "MENU",
	},

	{
		.keycode = KEY_F13,   // unused by hardware
		.desc = "F13",
	},

	{
		.keycode = KEY_F14,
		.desc = "F14",      // unused by hardware
	},
};


struct gpio_keyb_matrix_row spatha_matrix_rows[] = {
	{
		.buttons = spatha_row1_buttons,
		.gpio_output = AT91_PIN_PC23,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row2_buttons,
		.gpio_output = AT91_PIN_PC24,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row3_buttons,
		.gpio_output = AT91_PIN_PC25,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row4_buttons,
		.gpio_output = AT91_PIN_PC27,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row5_buttons,
		.gpio_output = AT91_PIN_PC30,
		.active_low_output = 1,
	},

	{
		.buttons = spatha_row6_buttons,
		.gpio_output = AT91_PIN_PC31,
		.active_low_output = 1,
	},

};

struct gpio_keyb_matrix_col spatha_matrix_cols[] = {
	{
		.gpio_input = AT91_PIN_PC22,
		.active_low_input = 1,
	},

	{
		.gpio_input = AT91_PIN_PC26,
		.active_low_input = 1,
	},

	{
		.gpio_input = AT91_PIN_PC28,
		.active_low_input = 1,
	},

	{
		.gpio_input = AT91_PIN_PC29,
		.active_low_input = 1,
	},
};
// --------------------------- END OF REV2
#endif
struct gpio_keyb_platform_data spatha_keyb_platform_data = {
		spatha_direct_buttons,
	ARRAY_SIZE(spatha_direct_buttons),
	spatha_matrix_rows,
	ARRAY_SIZE(spatha_matrix_rows),
	spatha_matrix_cols,
	ARRAY_SIZE(spatha_matrix_cols),
};

static struct platform_device spatha_keyb_device = {
	.name = "gpio-keyb",
	.id = 0,
	.dev = {
		.platform_data = &spatha_keyb_platform_data,
	},
	.num_resources = 0,
};

static void __init spatha_add_device_keyb(void)
{
	platform_device_register(&spatha_keyb_device);
}

#else
static void __init spatha_add_device_keyb(void) {}
#endif


/*
 * Framebuffer
 */
#if defined(CONFIG_FB_ATMEL) || defined(CONFIG_FB_ATMEL_MODULE)
// overrides similar function in at91sam9261_devices.c (wrong pinmux there)
extern struct atmel_lcdfb_info lcdc_data;
extern struct platform_device at91_lcdc_device;

void __init at91_add_device_lcdc_spatha(struct atmel_lcdfb_info *data)
{
	if (!data) {
		return;
	}

	at91_set_A_periph(AT91_PIN_PB0, 0);	/* LCDHSYNC */
	at91_set_A_periph(AT91_PIN_PB1, 0);	/* LCDHSYNC */
	at91_set_A_periph(AT91_PIN_PB2, 0);	/* LCDDOTCK */
	at91_set_A_periph(AT91_PIN_PB3, 0);	/* LCDDEN */

	at91_set_B_periph(AT91_PIN_PB5, 0);	/* LCDD0 */
	at91_set_B_periph(AT91_PIN_PB6, 0);	/* LCDD1 */
	at91_set_B_periph(AT91_PIN_PB7, 0);	/* LCDD2 */
	at91_set_B_periph(AT91_PIN_PB8, 0);	/* LCDD3 */
	at91_set_B_periph(AT91_PIN_PB9, 0);	/* LCDD4 */
	at91_set_B_periph(AT91_PIN_PB10, 0);	/* LCDD5 */
	at91_set_B_periph(AT91_PIN_PB11, 0);	/* LCDD6 */
	at91_set_B_periph(AT91_PIN_PB12, 0);	/* LCDD7 */
	at91_set_B_periph(AT91_PIN_PB15, 0);	/* LCDD10 */
	at91_set_B_periph(AT91_PIN_PB16, 0);	/* LCDD11 */
	at91_set_B_periph(AT91_PIN_PB17, 0);	/* LCDD12 */
	at91_set_B_periph(AT91_PIN_PB18, 0);	/* LCDD13 */
	at91_set_B_periph(AT91_PIN_PB19, 0);	/* LCDD14 */
	at91_set_B_periph(AT91_PIN_PB20, 0);	/* LCDD15 */

	lcdc_data = *data;
	platform_device_register(&at91_lcdc_device);
}
#endif

/*
 * Scanner head dynamic initialization
 */
static struct memory_accessor *at24_mem_acc;
static char readerhead[4] = {0, 0, 0, 0};

static void init_readerhead(char type[3]) {
	struct spi_master *spi_mstr;

	if (readerhead[0]) {
		printk(KERN_INFO "ERROR: Scanner head already initialized!\n");
		return;
	}

	if (type[0] < '0' || type[0] > '9' || type[1] < '0' || type[1] > '9'
			|| type[2] < '0' || type[2] > '9') {
		printk(KERN_INFO "ERROR: Bad scanner head type!\n");
		return;
	}

	readerhead[0] = type[0];
	readerhead[1] = type[1];
	readerhead[2] = type[2];
	printk(KERN_INFO "Scanner head type is %s.\n", readerhead);

	// conditionally load reader head modules:
	// RFID
	switch(readerhead[1]) {
#if defined(CONFIG_BOARD_RFID_NXP) || defined(CONFIG_BOARD_RFID_NXP_MODULE)
	case '1':
		printk(KERN_INFO "Enabling NXP HF RFID reader driver.\n");
		at91_set_B_periph(AT91_PIN_PC2, 1);		/* IRQ */
		spi_mstr = spi_busnum_to_master(1);
		if (spi_mstr)
			spi_new_device(spi_mstr, &spatha_nxp_spi_device);
		break;
#endif
#if defined(CONFIG_BOARD_RFID_AMS) || defined(CONFIG_BOARD_RFID_AMS_MODULE)
	case '2':
		printk(KERN_INFO "Enabling AMS UHF RFID reader driver.\n");
		at91_set_B_periph(AT91_PIN_PC2, 0);		/* IRQ */
		spi_mstr = spi_busnum_to_master(1);
		if (spi_mstr)
			spi_new_device(spi_mstr, &spatha_ams_spi_device);
		break;
#endif
	}
}

/* We set up a callback function for reader head I2C eeprom.
 * As the EEPROM gets detected, we read back the reader type from
 * it and initialize the required modules dynamically. */
static void at24_setup(struct memory_accessor *mem_acc, void *context) {
	char buf[6];

	at24_mem_acc = mem_acc;

	/* Read MAC addr from EEPROM */
	if (at24_mem_acc->read(at24_mem_acc, buf, 0x00, sizeof(buf)) == sizeof(buf)) {
		init_readerhead(buf + 1);
	} else {
		printk(KERN_INFO "Scanner head is not connected!\n");
	}
}


static struct at24_platform_data eeprom_info = {
	.byte_len = (1*1024) / 8,
	.page_size = 1,
	.flags = 0,
	.setup = at24_setup,
};

/* Manual initialization of the reader head via sysfs. This is needed
 * for factory procedure to avoid reboot after writing the eeprom.
 * As the head can be initialized only once, this can be used only
 * if the head is not already initialized by eeprom description. */
static struct bin_attribute reader_bin;
static ssize_t reader_bin_read(struct kobject *kobj, struct bin_attribute *attr,
		char *buf, loff_t off, size_t count)
{
	if (count < 4) return 0;
	if (off!=0) return 0;
	buf[0] = readerhead[0];
	buf[1] = readerhead[1];
	buf[2] = readerhead[2];
	buf[3] = readerhead[3];
	return 4;
}

static ssize_t reader_bin_write(struct kobject *kobj, struct bin_attribute *attr,
		char *buf, loff_t off, size_t count)
{
	if (count < 3) return count;
	init_readerhead(buf);
	return count;
}

/*
 * I2C
 */
#ifdef CONFIG_I2C_BOARDINFO
static struct i2c_board_info __initdata spatha_i2c_board_info[] = {
#ifdef CONFIG_MACH_SPATHA_REV2
	 {
			 I2C_BOARD_INFO("24c01", 0x50),
			 .platform_data  = &eeprom_info,
	 },
	 {
			 I2C_BOARD_INFO("isl29001", 0x44),
	 },
	 {
			 I2C_BOARD_INFO("lm75", 0x48),
			 .type="lm75a"
	 },
#endif
};
#define spatha_i2c_board_info_cnt (ARRAY_SIZE(spatha_i2c_board_info))
#else
#define spatha_i2c_board_info NULL
#define spatha_i2c_board_info_cnt 0
#endif


/*
 * power off the machine.
 */
static void spatha_power_off(void)
{
	void __iomem *sys_base = (void __iomem *) AT91_VA_BASE_SYS;
	__raw_writel(0xA5000001, sys_base + AT91_SHDW_CR );
	while (1);
}


static void __init spatha_board_init(void)
{
	pm_power_off = spatha_power_off;

	/* NOR flash */
	add_device_flash();

	/* Power control driver (power switching) */
	add_device_pwrctrl();

	/* Serial */
	at91_add_device_serial();

	/* USB Host */
	at91_add_device_usbh(&spatha_usbh_data);

	/* USB Device */
	at91_add_device_udc(&spatha_udc_data);

	/* I2C (temp sensor, vbat adc, light sensor) */
	at91_add_device_i2c(spatha_i2c_board_info, spatha_i2c_board_info_cnt);

	/* SPI (RFID) */
	at91_add_device_spi(spatha_spi_devices, ARRAY_SIZE(spatha_spi_devices));

	/* MMC (SD-card) */
	at91_add_device_mmc(0, &spatha_mmc_data);

	/* GPIO keyboard */
	spatha_add_device_keyb();

	/* LCDC */
	at91_add_device_lcdc_spatha(&spatha_lcdc_data);

	/* Watchdog timer driver */
	add_device_wdt();

	/* Speaker device */
	add_device_at91_spkr();

	/* Magstripe reader device */
	add_device_at91_mag();

	/* Touchscreen dev */
	spatha_add_device_ts();

	/* Bluetooth uart (exar) ttyS0 */
	add_device_serial_8250();

	/* Reader head manual initialization via sysfs */
	reader_bin.attr.name = "reader";
	reader_bin.attr.mode = S_IRUSR | S_IWUSR;
	reader_bin.read = reader_bin_read;
	reader_bin.write = reader_bin_write;

	sysfs_create_bin_file(power_kobj, &reader_bin);
}


MACHINE_START(AT91SAM9261EK, "Artec Design project spatha")
	.phys_io	= AT91_BASE_SYS,
	.io_pg_offst	= (AT91_VA_BASE_SYS >> 18) & 0xfffc,
	.boot_params	= AT91_SDRAM_BASE + 0x100,
	.timer		= &at91sam926x_timer,
	.map_io		= spatha_map_io,
	.init_irq	= spatha_init_irq,
	.init_machine	= spatha_board_init,
MACHINE_END
