
//--------------------------------------------------------------------------------


#define AMS_COMMAND				0x80
#define AMS_READ				0x40
#define AMS_CONTINUOUS			0x20

#define AMS_ADDRESS_MASK		0x1F

// AMS Chip Command Definitions

#define AMS_CMD_IDLE                  0x80	 // "Idle" command
#define AMS_CMD_SOFTINIT              0x83  // "Soft Init" command
#define AMS_CMD_HOPMAINFREQ           0x84  // "Hop to Main Frequency" command
#define AMS_CMD_HOPAUXFREQ            0x85  // "Hop to Aux Frequency" command
#define AMS_CMD_TRIGGERADCCON         0x87  // "Trigger ADC Conversion" command
#define AMS_CMD_TRIGGERRXFCALIBR      0x88  // "Trigger RX Filter Calibration" command
#define AMS_CMD_RESETFIFO             0x8F  // "Reset FIFO" command
#define AMS_CMD_TRANSMCRC             0x90  // "Transmit to Tag with CRC" command
#define AMS_CMD_TRANSMCRCEHEAD        0x91  // "Transmit to Tag with CRC,
                                                 // expect header bit" command
#define AMS_CMD_TRANSM                0x92  // "Transmit to Tag without CRC" command
#define AMS_CMD_DELAYEDTRANSM         0x93  // "Delayed Transmission without CRC"
#define AMS_CMD_DELAYEDTRANSMCRC      0x94  // "Delayed Transmission with CRC"
#define AMS_CMD_BLOCKRX               0x96  // "Block Receive" command
#define AMS_CMD_ENABLERX              0x97  // "Enable Receive" command
#define AMS_CMD_QUERY                 0x98  // "Send Query to Tag" command
#define AMS_CMD_QUERYREP              0x99  // "Send QueryRep to Tag" command
#define AMS_CMD_QUERYADJUSTUP         0x9A  // "Send Query Adjust Up" command
#define AMS_CMD_QUERYADJUSTNIC        0x9B  // "Send Query Adjust Nic" command
#define AMS_CMD_QUERYADJUSTDOWN       0x9C  // "Send Query Adjust Down" command
#define AMS_CMD_ACKN                  0x9D  // "Send Acknowledge" command
#define AMS_CMD_NAK                   0x9E  // "Send NOT Acknowledge" command
#define AMS_CMD_REQRN                 0x9F  // "Send Request RN" command

// AMS Chip Register Definitions

// Main Control
#define AMS_REG_STATUSCTRL            0x00  // Status Control
#define AMS_REG_PROTOCOLCTRL          0x01  // Protocol Control

// Protocol
#define AMS_REG_TXOPTGEN2             0x02  // Transmit Options GEN2
#define AMS_REG_RXOPTGEN2             0x03  // Receive Options GEN2
#define AMS_REG_TRCALREGGEN2          0x04  // TRcal low
#define AMS_REG_TRCALREGGEN2MISC      0x05  // TRcal and Misc
#define AMS_REG_TXREPLYINSLOT         0x06  // Transmition Reply Wait Time
#define AMS_REG_RXNORESPWAIT          0x07  // Wait Time until No Response
                                                 // Interrupt is generated
#define AMS_REG_RXWAITTIME            0x08  // Wait Time after Transmition
                                                 // until the Receiver is enabled
#define AMS_REG_RXSPECIAL             0x09  // Receive Special Settings
#define AMS_REG_RXSPECIAL2            0x0A  // Receive Special Settings 2
#define AMS_REG_REGIOCONTROL          0x0B  // Regulator and IO Control

#define AMS_REG_VERSIONREG            0x13  // Chip Version
#define AMS_REG_CLSYSAOCPCTRL         0x14  // CL_sys, analog out and CP control
#define AMS_REG_MODULATORCTRL         0x15  // Modulator Control
#define AMS_REG_PLLMAIN               0x16  // PLL R, A/B divider main register
#define AMS_REG_PLLAUX                0x17  // PLL A/B divider auxiliary register
#define AMS_REG_DAC                   0x18  // DAC control
#define AMS_REG_ADC                   0x19  // ADC control
#define AMS_REG_ADCSEL				  0x11	// ADC MUX

// Status
#define AMS_REG_IRQSTATUS             0x0C  // IRQ status
#define AMS_REG_IRQMASK               0x0D  // IRQ mask
#define AMS_REG_AGCINTERNALSTATUS     0x0E  // AGC and Internal status
#define AMS_REG_RSSILEVELS            0x0F  // RSSI levels
#define AMS_REG_AGLSTATUS             0x10  // AGL status

// Testregisters
#define AMS_REG_TESTSETTING           0x12

// FIFO registers
#define AMS_REG_RXLENGTHUP            0x1A  // number of bytes to receive (MSB)
#define AMS_REG_RXLENGTHLOW           0x1B  // number of bytes to receive (LSB)
#define AMS_REG_FIFOSTATUS            0x1C  // FIFO status
#define AMS_FIFOCNT_MASK		0x1F
#define AMS_FIFOOVF				0x20
#define AMS_REG_TXLENGTHUP            0x1D  // number of bytes to transmit (MSB)
#define AMS_REG_TXLENGTHLOW           0x1E  // number of bytes to transmit (LSB)
#define AMS_REG_FIFO                  0x1F  // data FIFO

#define TXLEN_H(by,bi)		(((by)>>4)&0xFF)
#define TXLEN_L(by,bi)		((((by)<<4)&0xF0) | (((bi)<<1)&0x0E) | (((bi)>0)?1:0))

// IRQ Mask register
#define IRQMASK_NORESP			0x01  // no response
#define IRQMASK_PREAMBLE		0x02  // preamble error
#define IRQMASK_RXCERR			0x04  // wrong number of rx bits
#define IRQMASK_HEADER			0x08  // high header bit
#define IRQMASK_CRCERR			0x10  // CRC error
#define IRQMASK_FIFO			0x20  // FIFO needs attention

// all required bits (except FIFO, FIFO handling with IRQs seems broken)
#define IRQMASK_ALL				(IRQMASK_NORESP | IRQMASK_PREAMBLE | IRQMASK_RXCERR | IRQMASK_HEADER | IRQMASK_CRCERR)
// all required bits except NORX - enabling NORX IRQ disables receiver on NORX event
#define IRQMASK_NORXTIMEOUT		(IRQMASK_PREAMBLE | IRQMASK_RXCERR | IRQMASK_HEADER | IRQMASK_CRCERR)

// IRQ STATUS register
#define IRQ_TX						0x80  // transmit complete
#define IRQ_RX						0x40  // receive complete
#define IRQ_FIFO					0x20  // signals FIFO high or low
#define IRQ_CRCERR					0x10  // CRC error
#define IRQ_HDR						0x08  // received header bit is high
#define IRQ_RXCERR					0x04  // wrong number of received bytes
#define IRQ_PREAMBLEERR				0x02  // error during preamble reception
#define IRQ_NORX					0x01  // no response bit

// Software pseudo-IRQ's
#define SIRQ_OVF					0x100	// FIFO RX buffer overflow
#define SIRQ_UNF					0x200	// FIFO TX buffer underflow
#define SIRQ_GEN					0x400	// Other errors
#define SIRQ_FIFORST_DONE			0x1000	// Fifo has been reset after TX int

#define IRQ_ERR				(IRQ_CRCERR | IRQ_RXCERR | IRQ_PREAMBLEERR | IRQ_NORX | SIRQ_UNF | SIRQ_OVF | SIRQ_GEN)
#define IRQ_ERR_NORX		(IRQ_CRCERR | IRQ_RXCERR | IRQ_PREAMBLEERR | SIRQ_UNF | SIRQ_OVF | SIRQ_GEN)
#define IRQ_ERR_NOTIMEOUT	(IRQ_CRCERR | IRQ_RXCERR | /*IRQ_PREAMBLEERR |*/ SIRQ_UNF | SIRQ_OVF | SIRQ_GEN)
#define IRQ_DONE			(IRQ_ERR | IRQ_RX)

// FIFO STATUS register
#define FIFOSTATUS_HIGH				0x40  // FIFO 2/3 full
#define FIFOSTATUS_LOW				0x20  // FIFO 1/3 full
#define FIFOSTATUS_OVERFLOW			0x10  // FIFO overflow

#define FIFO_SIZE				24
#define FIFO_TXIRQ_LEVEL		6
#define FIFO_RXIRQ_LEVEL		18
#define FIFO_FILL_TXIRQ			(FIFO_SIZE - FIFO_TXIRQ_LEVEL)
#define FIFO_READ_RXIRQ			(FIFO_RXIRQ_LEVEL)

#define XBUF_CNT					128		// extra tx/rx data buffer

// number of tries to write until failure is declared
#define TRIES_WRITE				5
#define TRIES_WRITE_FIRST		2

// #define USE_FIFO_IRQ // do NOT use fifo loading, it is broken in chip v3.8!

//--------------------------------------------------------------------------------
// Structures, global variables

#ifdef USE_FIFO_IRQ
struct rfid_ams_spidata_irqfifo {
	char tx[FIFO_SIZE+2];
	char rx[FIFO_SIZE+2];
};
#endif

struct rfid_ams_spidata_irqflags {
	char tx[2];
	char rx[2];
};

// Function type that will be used as callback from ams layer.
// If a async call is initiated, set ams_async_handler as callback function
// with context as argument and return 0.
// If transfer is initiated, set ams_enable_irq_unlock as callback and
// return 0.
// On error return non-zero.
typedef int(*cmd_t)(void);

struct rfid_ams {
  struct rfid_ams_plat_data *pdata;     // platform data that specifies the GPIO pins
  int chip_version;
#define CHIP_AS3991 0x35
#define CHIP_AS3992	0x50
  int ref_freq;
  int irq_disabled;

  // SPI
  struct spi_device      *p_spi_dev;
  struct spi_message     spi_mess, spi_mess_irqflags, spi_mess_irqfifo;
  struct spi_transfer    spi_trans, spi_trans_irqflags, spi_trans_irqfifo;
  char *tx_buf;
  char *rx_buf;
  struct rfid_ams_spidata_irqflags *irqflags_buf;

  // IRQ-main communication
  int irq_flags;	// cumulative irq flags
  cmd_t *irq_cmd_chain;	// NULL-terminated list of command chains to be performed on RX irq
  int exit_irq;		// exit_irq flag - if this is set, abort all async processing
  int irq_async;	// flag to indicate if async processing is running
  int irq_done;		// stop async processing when one of these flags is set and irq_cmd<=0
  int irq_err;		// stop async processing with error when one of these flags is set

  int rn16;

  // suspend-main communication
  int stop_reading;		// set by suspend to indicate that read should exit
  int suspend_powerup;	// should the reader be powered back up on resume

  // user settings
  int dataStart, dataEnd;	// data range to read from tag (in 16bit blocks)
  unsigned char dataBank;	// EPC memory bank to read from

  // HW settings
  int flags;				// flags from board definition, list in rfid_ams.h

  // tx/rx timing settings
  int m;					// 2^m = cycles per symbol (Miller encoding)
  int rxlinkfreq;			// rx link frequency
#define RXLINKFREQS	{40000, 80000, 160000, 213333, 256000, 320000, 640000} // Hz
  int dr;					// rx link division ratio {8, 64/3}
  int tari;					//
#define TARIS {6250, 12500, 25000} //ns
#define FT 20				// frequency tolerance, in percents
  int trext;				// extra pilot tone
  int pilotreduce;			// shorten pilot tone measurement (AS3992)

  // rx gain settings
  signed char rxgain;
  char rxhyst;
  char autogain; // 0: off; 1: AGC; 2: AGL
  char rxflags;
#define RX_MIXATTEN8	0x01	// Mixed input attenuation -8dB(diff)/-5dB(single)
#define RX_MIXAMPL10	0x02

  // frequency hopping
  int band;					// frequency band
#define NUM_BANDS	3
#define FREQ_EU		0
#define FREQ_US		1
#define FREQ_JP		2
  int fixed_channel;		// fixed channel number, if invalid then hopping
  int channel;				// current channel, -1 if PLL disabled
  unsigned int lfsr;		// channel hopping LFSR

  // reg cache
  unsigned char r05, r0a;
};

extern struct rfid_ams info;
extern wait_queue_head_t irq_wait_queue;
extern struct file_operations rfid_ams_fops;

//--------------------------------------------------------------------------------
// Functions

unsigned char ams_read(unsigned char addr);
int ams_read_cont(unsigned char addr, char *buf, int count);
// Do not use ams_read_cont for registers 0x14-0x1E, it does not work well

#define ams_write(a,d) ams_write_async(a,d,NULL)
int ams_write_async(unsigned char addr, unsigned char data,
		void (*complete)(void *));
#define ams_write_cont(a,b,c) ams_write_cont_async(a,b,c,NULL)
int ams_write_cont_async(unsigned char addr, char *buf, int count,
		void (*complete)(void *));
#define ams_command(c) ams_command_async(c,NULL)
int ams_command_async(unsigned char cmd,
		void (*complete)(void *context));
#define ams_command_write(cm,a,d) ams_command_write_async(cm,a,d,NULL)
int ams_command_write_async(unsigned char cmd,
		unsigned char address, unsigned char data,
		void (*complete)(void *context));
#define ams_command_write_cont(cm,a,b,c) ams_command_write_cont_async(cm,a,b,c,NULL)
int ams_command_write_cont_async(unsigned char cmd,
		unsigned char address, unsigned char *buf, unsigned char buf_len,
		void (*complete)(void *context));

int ams_read_fifo(char *dbuf, int count);

void ams_async_handler(void *context);
int ams_cmd_resetfifo(void);
int ams_cmd_read_rn16(void);

void ams_on(void);
void ams_set_next_frequency(void);
void ams_set_frequency(unsigned long frequency);
int ams_get_freq(int band, int channel);
void ams_set_rxgain(signed char gain);
void ams_configure(void);
void ams_off(void);

void ams_prepare_irq(void);
int ams_wait_irq_done(int timeout);
void ams_enable_irq(void *context);
void ams_enable_irq_unlock(void *context);
void ams_disable_irq_sync(void);

