#ifndef PWRCTRL_H_
#define PWRCTRL_H_

// IOCTL interface: (cmd, arg)
// GPIO OUTPUT PINS
#define PWRCTRL_SET_GPIO 0xED00		// arg: uchar pin_id
#define PWRCTRL_SET_GPIO_HI (PWRCTRL_SET_GPIO | 0x0001)
#define PWRCTRL_SET_GPIO_LO (PWRCTRL_SET_GPIO | 0x0000)

// GPIO INPUT PINS
#define PWRCTRL_GET_GPIO 0xED10		// arg: uchar pin_id

// KEYBOARD backlight
#define PWRCTRL_SET_KBDBL 0xED24	// arg: uchar value
#define PWRCTRL_FADE_KBDBL_TO 0xED25	// arg: uchar value


#define PWRCTRL_USART_TRISTATE_ON 0xED31		// arg: uchar usart
#define PWRCTRL_USART_TRISTATE_OFF 0xED30		// arg: uchar usart

#define PWRCTRL_USART_GPRS 1
#define PWRCTRL_USART_BARCODE 2

#define KBDBL_LEVEL_MAX			0xff

// GPIO PIN DEFINITIONS
	// System generic
#define O_PIN_LED_R 		0
#define O_PIN_LED_G 		1

	// EXT BOARDS
#define O_PIN_EXT3VEN_N 	2
#define O_PIN_EXT5VEN_N 	3
#define O_PIN_MPCIE3VEN 	4

	// GPRS
#define O_PIN_GPRS_PWR 		5 // GPRS VBAT power
#define O_PIN_GPRS_RST		6 // GPRS Reset
#define O_PIN_GPRS_ON		7 // GPRS ON signal
#define O_PIN_GPRS_DTR		8 // flow control: DTR, used for power saving

	// WLAN
#define O_PIN_WLAN_RADIO_ON	7 // WLAN radio ON signal

	// BT
#define O_PIN_BT_PWR		4
#define O_PIN_BT_RST		6

	// 1D
#define O_PIN_1D_AIM		9
#define O_PIN_1D_PWR_N  O_PIN_EXT3VEN_N

#define O_PIN_COUNT			10

	// System generic
#define	I_PIN_USB_VBUS		0 // USB DEV power present
#define	I_PIN_LED_R			1 // CHARGING
#define	I_PIN_LED_G			2 // CHARGING COMPLETE

	// GPRS
#define	I_PIN_GPRS_PWRGD	3

	// 2D
#define	I_PIN_1D_PWR_DWN	4

	// keypad
#define	I_PIN_BTN_POWER		5

#define	I_PIN_COUNT			6


// DRIVER PARAMETER STRUCTURES, FOR BOARD INITIALIZATION
enum gpio_pin_controllers {
	CTRL_GPIO, //CPU-connected GPIO pins
	CTRL_GPIO_OD, //CPU-connected GPIO pins, emulate open-drain
};


struct gpio_input_pin {
	int controller; // controller type, from enum gpio_pin_controllers
	int pin; // pin ID within that controller
};

struct gpio_output_pin {
	int controller; // controller type, from enum gpio_pin_controllers
	int pin; // pin ID within that controller
	int def; // default state, set on load
};


struct pwrctrl_platform_data {
	struct gpio_input_pin input_pins[I_PIN_COUNT];
	struct gpio_output_pin output_pins[O_PIN_COUNT];
	int kbbl_en_pin; // keyboard backlight enable pin
};

#endif /*PWRCTRL_H_*/
