/* include/linux/rfid_ams.h
 *
 * Copyright (c) 2010 Artec Design LLC
 *   Jaanus Sepp <jaanus.sepp@artecdesign.ee>
 *   Anti Sullin <anti.sullin@artecdesign.ee>
 *
 * Header file for Austria Microsystems RFID Reader platform data
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
*/

#ifndef __RFID_AMS_PLATFORM_DATA
#define __RFID_AMS_PLATFORM_DATA __FILE__

enum enum_rfid_ams_pins {
  RFID_AMS_PIN_ENABLE,
  RFID_AMS_PIN_PWR_N,
  RFID_AMS_PINCOUNT
};

struct rfid_ams_plat_data {
  unsigned int pin_assignment[RFID_AMS_PINCOUNT];
  unsigned int irq;
  unsigned int flags;
#define AMS_EXTERNAL_PA			0x01
#define AMS_EXTERNAL_OSC		0x02
#define AMS_DIFF_INPUT			0x04
};

// ioctl definitions:
// stop reading a blocking read
#define RFID_AMS_STOP_READ				0xef01

// select EPC data to read in addition to EPC id.
// if block_start < block_end, data will be read according to these settings
#define RFID_AMS_SET_EPC_BANK			0xf001
#define RFID_AMS_SET_BLOCK_START		0xf002
#define RFID_AMS_SET_BLOCK_END			0xf003

// set band and channel
// arg bits0-15: band;
// arg bits16-31: channel no; 0xFFFF if hopping
#define RFID_AMS_SET_CHAN				0xf101
// get current band and channel (real-time value)
#define RFID_AMS_GET_CHAN				0xf102
// get current frequency
#define RFID_AMS_GET_FREQ				0xf103

// get RSSI level of last transmission
// bits 0...7	rssi
// bit	8		source (0: I, 1: Q)
// bits 16...19 AGC status
#define RFID_AMS_GET_RSSI				0xf201
// get mixer DC levels for signal reflection level calculation
#define RFID_AMS_GET_RDC				0xf202

// set / get RX/TX parameters
// bits 0...1	tari {6250, 12500, 25000} ns
// bits 8...10	rxlinkfreq {40000, 80000, 160000, 213333, 256000, 320000, 640000} Hz
// bit  16		dr {8, 64/3}
#define RFID_AMS_SET_RXTXPARAMS			0xf301
#define RFID_AMS_GET_RXTXPARAMS			0xf302

// set / get RX modulation parameters
// bits 0...1	m {FM0, M2, M4, M8}
// bit  8		trext {extra pilot tone} (set to 0 only if m==[M2|M4])
// bits 16...17	reduce pilot measurement time (AS3992 only)
#define RFID_AMS_SET_RXMODPARAMS		0xf311
#define RFID_AMS_GET_RXMODPARAMS		0xf312

// set / get RX amp gain parameters
// bits 0...7	rx gain, signed char
// bits 8...10	rx hysteresis
// bits 16..17	rx autogain {off, AGC, AGL}
// bit  24		dif mixer -8dB(diff)/-5dB(single-ended) attenuation
// bit  25		dif mixer +10dB (AS3992 only)
#define RFID_AMS_SET_RXGAINPARAMS		0xf321
#define RFID_AMS_GET_RXGAINPARAMS		0xf322

struct rfid_ams_hdr {
	unsigned char ret;
	unsigned char idLen;
	unsigned short dataLen;
} __attribute__((packed));

struct rfid_ams_write_hdr {
	unsigned char bank;
	unsigned short offset; // in blocks
	unsigned short len; // in blocks
} __attribute__((packed));



#endif /* __RFID_AMS_PLATFORM_DATA */

